package PowerListingFieldEditor::CMS;

use strict;

use MT;
use MT::ListProperty;
use MT::Util;
use MT::Util qw( archive_file_for first_n_words encode_html epoch2ts format_ts relative_date encode_url );
use CustomFields::Field;
use CustomFields::App::CMS;

use PowerListingFieldEditor::CMS::Asset;
use Data::Dumper;

sub init_request {
    my ($eh, $app) = @_;

    my $plugin = $app->component("PowerListingFieldEditor");
    my $field_column = $plugin->get_config_value('field_column', 'system');
    if (!$field_column) {
        $plugin->set_config_value('field_column', time(), 'system');
    }
    $plugin->set_config_value('rest_day', rest_day(), 'system');
    return if is_limit();

    my $q = $app->param;
    my $mode = $q->param('__mode');

    my $blog_id = $q->param('blog_id');
    return if !$blog_id;

    return if !($mode eq 'list' || $mode eq 'filtered_list' || $mode eq 'save_from_list' || $mode eq 'save_from_list_cd');

    my $type = $q->param('_type') || $app->param('datasource');
#    return if !($type eq 'entry' || $type eq 'page' || $type eq 'comment' || $type eq 'asset' || $type eq 'member');
    return if !($type eq 'entry' || $type eq 'page' || $type eq 'content_data');

    my $blog_id = $q->param('blog_id') || return;

    my $blog = MT::Blog->load($blog_id);
    return if !$blog;
#    return if $blog->class eq 'website';

make_content_actions();

    my $asset_type = [qw( file audio image video)] if $type eq 'asset';

    my @data1 = [
        {   class   => 'status2',
            label   => 'Status',
            display => 'default',
        },
        {   class   => 'view-link2',
            label   => 'Link',
            display => 'default',
        },
    ];

    my $plugin = $app->component('PowerListingFieldEditor');
    my $prop;

    my $order = 100;

    $prop->{status3} = {
        label     => 'Status3',
        col       => 'status',
        display   => 'optional',
        html      => \&out_html_default_status,
        col_class => 'icon',
        order     => $order += 10,
        base      => '__virtual.single_select',
        single_select_options => [
              { label => MT->translate('Draft'),     value => 1, },
              { label => MT->translate('Published'), value => 2, },
              { label => MT->translate('Reviewing'), value => 3, },
              { label => MT->translate('Scheduled'), value => 4, },
              { label => MT->translate('Junk'),      value => 5, },
        ],
        filter_editable => 0
    };

    $prop->{"blog${blog_id}_title"} = {
        label => 'title2',
        col => 'title',
        column =>  'title2',
        display => 'default',
        html => \&out_html_default_title,
        bulk_sort => \&bulk_sort_default,
        order => $order += 10,
        sub_fields => @data1,
        filter_tmpl    => '<mt:var name="filter_form_string">',
        terms     => sub {
            my $prop = shift;
            my ( $args, $db_terms, $db_args ) = @_;
            my $col    = $prop->col or die;
            my $option = $args->{option};
            my $query  = $args->{string};
            if ( 'contains' eq $option ) {
                $query = { like => "%$query%" };
            }
            elsif ( 'not_contains' eq $option ) {
                $query = { not_like => "%$query%" };
            }
            elsif ( 'beginning' eq $option ) {
                $query = { like => "$query%" };
            }
            elsif ( 'end' eq $option ) {
                $query = { like => "%$query" };
            }
            if ( $prop->is_meta ) {
                return $prop->join_meta( $db_args, $query );
            }
            else {
                return { $col => $query };
            }
        },

    };

    foreach my $col (qw( text excerpt text_more keywords tags )) {
        my $key = "blog${blog_id}_".$col;
        $prop->{$key} = {
            label => $plugin->translate($col),
            column =>  $col,
            display => $col eq 'authored_on' ? 'default' : 'optional',
            html => \&out_html_default,
            bulk_sort => \&bulk_sort_default,
            type => $col =~ /text|excerpt/ ? 'textarea' :
                    $col =~ /_on/ ? 'date' : 'text',
            order => $order += 10,
        };
    }

    $prop->{"blog${blog_id}_authored_on"} = {
        label      => $plugin->translate('authored_on'),
        column     => 'authored_on',
        display    => 'optional',
        html       => \&out_html_default,
        bulk_sort  => \&bulk_sort_default,
        default_sort_order => 'descend',
        col_class     => 'date',
        type => 'date',
        order      => $order += 10,
    };

    $prop->{"blog${blog_id}_modified_on"} = {
        label      => $plugin->translate('modified_on'),
        column     => 'modified_on',
        display    => 'optional',
        html       => \&out_html_default,
        bulk_sort => \&bulk_sort_default,
        col_class     => 'date',
        type => 'date',
        order      => $order += 10,
    };

    $prop->{status2} = {
        label     => 'Status2',
        col       => 'status',
        display   => 'none',
        col_class => 'icon',
        base      => '__virtual.single_select',
        single_select_options => [
              { label => MT->translate('Draft'),     value => 1, },
              { label => MT->translate('Published'), value => 2, },
              { label => MT->translate('Reviewing'), value => 3, },
              { label => MT->translate('Scheduled'), value => 4, },
              { label => MT->translate('Junk'),      value => 5, },
        ],
        filter_editable => 0
    };

    my @category_list;
    my $class;
    if ( $type eq 'entry' ) {
        $class = MT->model('category');
    } elsif ( $type eq 'page' ) {
        $class = MT->model('folder');
    }

if ( $type ne 'content_data' ) {
    my @categories = $class->load( { blog_id => $blog_id } );
    for my $category (@categories) {
        push @category_list, { label => $category->label, value => $category->id };
    }

    $prop->{category2} = {
        label     => $type eq 'entry' ? 'Category2' : 'Folder',
        col       => 'category',
        display   => 'optional',
        html      => \&out_html_default_category,
        col_class => 'icon',
        view_filter      => 'blog',
        category_class   => $type eq 'entry' ? 'category' : 'folder',
        order     => $order += 10,
        base      => '__virtual.single_select',
        single_select_options => \@category_list,

        terms => sub {
            my ( $prop, $args, $db_terms, $db_args ) = @_;
            my $blog_id = MT->app->blog->id;
            my $app     = MT->instance;
            my $option  = $args->{option};

            my $category = MT->model( $prop->category_class )->load( $args->{value} );
            $args->{string} = $category->label;

            my $query   = $args->{string};
            push @{ $db_args->{joins} }, MT->model('placement')->join_on(
                undef,
                {   entry_id => \'= entry_id',
                    blog_id  => $blog_id,
                },
                {   unique => 1,
                    join   => MT->model( $prop->category_class )->join_on(
                        undef,
                        {   label   => $query,
                            id      => $type eq 'entry' ? \'= placement_category_id' : \'= placement_folder_id',
                            blog_id => $blog_id,
                        },
                        { unique => 1, }
                    ),
                },
            );
            return;
        },
    };
}
    my $asset_key = "blog${blog_id}_include_asset_ids";
    $prop->{$asset_key} = {
        label   => 'Asset',
        col     => 'include_asset_ids',
        display => 'optional',
        html    => \&out_html_default_asset,
        type    => 'image',
        order   => $order += 10,
    };
##################

    my @fields = CustomFields::Field->load({
        obj_type => ($asset_type) ? $asset_type : $type eq 'member' ? 'author' : $type,
        blog_id => $blog_id ? [ $blog_id, 0 ] : 0,
    });
#    return if !@fields;

    my @fields_image;
    my $order = 1000;
    foreach my $field (@fields) {
        if (!( $field->type eq 'text' ||
                   $field->type eq 'textarea' ||
                   $field->type eq 'checkbox' ||
                   $field->type eq 'select' ||
                   $field->type eq 'radio' ||
                   $field->type eq 'url' )) {
            push @fields_image, $field;
            next;
        }
        my $key = "blog${blog_id}_".$field->basename;
        $prop->{$key} = {
            label => $field->name,
            display => 'optional',
            html => \&out_html,
            bulk_sort => \&bulk_sort,
            type => $field->type,
            basename => $field->basename,
            order => $order += 100,
            option => $field->options ? ($field->options) : '',
        };
    }
    foreach my $field (@fields_image) {
        next if !( $field->type eq 'image' );
        my $key = "blog${blog_id}_".$field->basename;
        $prop->{$key} = {
            label => $field->name,
            display => 'optional',
            html => \&out_html,
            type => $field->type,
            basename => $field->basename,
            order => $order += 100,
        };
    }
    return 1 if !$prop;

    $plugin->registry({
        list_properties => {
            $type => $prop,
        },
    });

    my $plugin = MT->component("PowerListingFieldEditor");
    my $field_column = $plugin->get_config_value('field_column', 'system');
    if (!$field_column) {
        $plugin->set_config_value('field_column', time(), 'system');
    }

    my $core = MT->component("core");
#    $core->registry("applications","cms","content_actions","entry","reset_all_counter",
#        {
#            class => 'icon-action',
#            id    => 'action-edit-all',
#            label => $plugin->translate('Edit all'),
#            order => 100,
#        });

#    my $core = MT->component("core");
    $core->registry("list_properties","entry","title",
        { display => 'optional' });
    $core->registry("list_properties","entry","authored_on",
        { 
            auto       => 1,
             display   => 'none',
            label      => 'Publish Date',
            use_future => 1,
            order      => 600,
        });
    $core->registry("list_properties","entry","modified_on",
        { base  => '__virtual.modified_on', display => 'none' });
    $core->registry("list_properties","entry","category",
        { display => 'none' });

    $core->registry("list_properties","entry","id",
        { default_sort_order  => 'descend',
          base  => '__virtual.id',
          order => 100,
        });
    $core->registry("list_properties","page","id",
        { default_sort_order  => 'descend',
          base  => '__virtual.id',
          order => 100,
        });

    $core->registry("listing_screens","entry",
        { default_sort_key => "blog${blog_id}_authored_on" });
    $core->registry("listing_screens","page",
        { default_sort_key => "blog${blog_id}_authored_on" });

    1;
}

sub make_list_props {
    my $props = {};

    for my $content_type ( @{ MT::ContentType->load_all } ) {
        my $key   = 'content_data.content_data_' . $content_type->id;
        my $order = 1000;
        my $field_list_props
            = _make_field_list_props( $content_type, $order );

        if ( $order < 2000 ) {
            $order = 2000;
        }
        else {
            $order = ( POSIX::floor( $order / 100 ) + 1 ) * 100;
        }

        $props->{$key} = {
            id => {
                base    => '__virtual.id',
                display => 'optional',
                order   => 100,
            },
            label => {
                base    => '__virtual.string',
                display => 'force',
                col     => 'label',
                order   => 110,
                html    => \&_make_label_html,
                label   => sub {
                    MT->translate('Data Label');
                },
                bulk_sort => sub {
                    my $prop = shift;
                    my ( $objs, $app, $opts ) = @_;
                    return
                        sort { ( $a->label || '' ) cmp( $b->label || '' ) }
                        @$objs;
                },
                terms => sub {
                    my $prop = shift;
                    my ( $args, $db_terms, $db_args ) = @_;

                    my $ct
                        = MT->model('content_type')
                        ->load( $db_terms->{content_type_id} )
                        or die MT->translate(
                        'Cannot load content type #[_1]',
                        $db_terms->{content_type_id}
                        );
                    if ( !$ct->data_label ) {

                        # Use __virtual.string based filtering
                        my $code
                            = MT->handler_to_coderef( $prop->base->{terms} );
                        return $code->( $prop, @_ );
                    }
                    else {
                        # Filter by field value
                        my $option = $args->{option};
                        my $query  = $args->{string};

                        if ( 'contains' eq $option ) {
                            $query = { like => "%$query%" };
                        }
                        elsif ( 'not_contains' eq $option ) {
                            $query
                                = [ { not_like => "%$query%" }, \'IS NULL' ];
                        }
                        elsif ( 'beginning' eq $option ) {
                            $query = { like => "$query%" };
                        }
                        elsif ( 'end' eq $option ) {
                            $query = { like => "%$query" };
                        }
                        elsif ( 'blank' eq $option ) {
                            $query = [ \'IS NULL', '' ];
                        }
                        elsif ( 'not_blank' eq $option ) {
                            $query
                                = [ '-and', \'IS NOT NULL', { not => '' } ];
                        }

                        my $cf
                            = MT->model('content_field')
                            ->load( { unique_id => $ct->data_label, } )
                            or die MT->translate(
                            'Cannot load content field #[_1]',
                            $ct->data_label );

                        $db_args->{joins} ||= [];
                        push @{ $db_args->{joins} },
                            MT->model('content_field_index')->join_on(
                            undef,
                            [   { content_data_id => \'= cd_id' },
                                [   { value_varchar => $query },
                                    '-or',
                                    { value_text => $query },
                                ],
                            ],
                            {   join => MT->model('content_field')->join_on(
                                    undef,
                                    {   id => \'= cf_idx_content_field_id',
                                        unique_id => $ct->data_label,
                                    },
                                ),
                                unique => 1,
                            },
                            );
                    }
                },
                sub_fields => [
                    {   class   => 'status',
                        label   => 'Status',
                        display => 'default',
                    },
                    {   class   => 'view-link',
                        label   => 'Link',
                        display => 'default',
                    },
                ],
            },
            author_name => {
                base  => '__virtual.author_name',
                order => $order,
            },
            authored_on => {
                auto    => 1,
                display => 'default',
                label   => sub {
                    MT->translate('Publish Date');
                },
                use_future => 1,
                order      => $order + 100,
                sort       => sub {
                    my $prop = shift;
                    my ( $terms, $args ) = @_;
                    my $dir = delete $args->{direction};
                    $dir = ( 'descend' eq $dir ) ? "DESC" : "ASC";
                    $args->{sort} = [
                        { column => $prop->col, desc => $dir },
                        { column => "id",       desc => $dir },
                    ];
                    return;
                },
            },
            created_on => {
                base  => '__virtual.created_on',
                order => $order + 200,
            },
            modified_on => {
                base  => '__virtual.modified_on',
                order => $order + 300,
            },
            unpublished_on => {
                auto    => 1,
                display => 'optional',
                label   => sub {
                    MT->translate('Unpublish Date');
                },
                order => $order + 400,
            },
            status    => { base => 'entry.status' },
            author_id => {
                base            => 'entry.author_id',
                label_via_param => sub {
                    my $prop = shift;
                    my ( $app, $val ) = @_;
                    my $author = MT->model('author')->load( $val || 0 )
                        or return $prop->error(
                        MT->translate(
                            '[_1] ( id:[_2] ) does not exists.',
                            MT->translate("Author"),
                            defined $val ? $val : ''
                        )
                        );
                    return MT->translate( 'Contents by [_1]',
                        $author->nickname );
                },
            },
            author_status => { base    => 'entry.author_status' },
            blog_name     => { display => 'none', filter_editable => 0 },
            current_context => { filter_editable => 0 },
            %{$field_list_props},
        };
        if ( $content_type->_get_tag_field_ids ) {
            $props->{$key}{tags_field} = {
                base  => '__virtual.tag',
                label => sub {
                    MT->translate('Tags fields');
                },
                display         => 'none',
                filter_editable => 0,
                use_blank       => 1,
            };
        }
    }

    return $props;
}

sub _make_field_list_props {
    my ( $content_type, $order, $parent_field_data ) = @_;
    my $props               = {};
    my $content_field_types = MT->registry('content_field_types');

    for my $field_data ( @{ $content_type->fields } ) {
        my $idx_type   = $field_data->{type};
        my $field_key  = 'content_field_' . $field_data->{id};
        my $field_type = $content_field_types->{$idx_type} or next;

        for my $prop_name ( keys %{ $field_type->{list_props} || {} } ) {
            next if $prop_name eq 'plugin';
            next
                if $parent_field_data
                && $prop_name ne $idx_type
                && !( $idx_type eq 'content_type' && $prop_name eq 'id' );

            my $label;
            if ( $prop_name eq $idx_type ) {
                $label = $field_data->{options}{label};
            }
            else {
                $label = $prop_name;
                if ( $label eq 'id' ) {
                    $label = 'ID';
                }
                else {
                    $label =~ s/^([a-z])/\u$1/g;
                    $label =~ s/_([a-z])/ \u$1/g;
                }
                $label = MT->translate($label);
                $label = $field_data->{options}{label} . " ${label}";
            }
            if ($parent_field_data) {
                $label = $parent_field_data->{options}{label} . " ${label}";
            }

            my $prop_key;
            if ( $prop_name eq $idx_type ) {
                $prop_key = $field_key;
            }
            else {
                $prop_key = "${field_key}_${prop_name}";
            }
            if ($parent_field_data) {
                my $parent_field_key
                    = 'content_field_' . $parent_field_data->{id};
                $prop_key = "${parent_field_key}_${prop_key}";
            }

            my $display
                = $parent_field_data
                ? 'none'
                : 'optional'; # $field_data->{options}{display};
#MT->instance->log($idx_type);
            $props->{$prop_key} = {
                (   content_field_id   => $field_data->{id},
                    data_type          => $field_type->{data_type},
                    default_sort_order => 'ascend',
                    display            => $display,
                    filter_label       => $label,
#                    html               => \&_make_title_html,
                    html               => \&out_html_cd,
bulk_sort => \&bulk_sort_cd,
                    idx_type           => $idx_type,
                    label              => $label,
                    order              => $order,
                    sort               => \&_default_sort,
                ),
                %{ $field_type->{list_props}{$prop_name} },
            };

            if ($parent_field_data) {
                my $terms = $props->{$prop_key}{terms};
                if ( $terms && !ref $terms && $terms =~ /^(sub|\$)/ ) {
                    $terms = MT->handler_to_coderef($terms);
                    $props->{$prop_key}{terms} = sub {
                        my $prop = shift;
                        my ( $args, $db_terms, $db_args ) = @_;

                        my $child_ret;
                        {
                            local $db_terms->{content_type_id}
                                = $content_type->id;
                            $child_ret = $terms->( $prop, @_ );
                        }
                        return $child_ret
                            unless $child_ret && $child_ret->{id};

                        local $prop->{content_field_id}
                            = $parent_field_data->{id};

                        my $option = $args->{option} || '';
                        if (   $option eq 'not_contains'
                            || $option eq 'not_equal' )
                        {
                            my $cd_terms;
                            if ( ref $child_ret->{id} eq 'HASH'
                                && $child_ret->{id}{not} )
                            {
                                $cd_terms = { id => $child_ret->{id}{not} };
                            }
                            else {
                                $cd_terms
                                    = { id => { not => $child_ret->{id} } };
                            }
                            my @child_contains_cd_ids
                                = map { $_->id }
                                MT::ContentData->load( $cd_terms,
                                { fetchonly => { id => 1 } } );
                            my $join_terms = { value_integer =>
                                    [ \'IS NULL', @child_contains_cd_ids ] };
                            my $cd_ids = get_cd_ids_by_left_join( $prop,
                                $join_terms, undef, @_ );
                            $cd_ids ? { id => { not => $cd_ids } } : ();
                        }
                        else {
                            my $join_terms
                                = { value_integer => $child_ret->{id} };
                            my $cd_ids = get_cd_ids_by_inner_join( $prop,
                                $join_terms, undef, @_ );
                            { id => $cd_ids };
                        }
                    };
                }
            }

            $order++;
        }

        if ( !$parent_field_data && $idx_type eq 'content_type' ) {
            my $cf = MT::ContentField->load( $field_data->{id} ) or next;
            my $related_ct = $cf->related_content_type or next;
            my $child_props
                = _make_field_list_props( $related_ct, $order, $field_data );
            $props = { %{$props}, %{$child_props} };
        }
    }

    $_[1] = $order;
#MT->instance->log(Dumper($props));
    $props;
}

sub _make_title_html {
    my ( $prop, $content_data, $app ) = @_;

    my $label = $content_data->data->{ $prop->content_field_id };
    if ( $label && ref $label eq 'ARRAY' ) {
        my $delimiter = $app->registry('content_field_types')
            ->{ $prop->{idx_type} }{options_delimiter} || ',';
        $label = join "${delimiter} ", @$label;
    }

    $label = '' unless defined $label;
    my $can_double_encode = 1;
    $label = MT::Util::encode_html( $label, $can_double_encode );

    return qq{<span class="label">$label</span>};
}

sub post_init {
    my ($eh, $app) = @_;

    my $core = MT->component("core");
    my $props = make_list_props();

    my $core_list_props = $core->registry('list_properties');
    for my $key ( keys %{$props} ) {
        $core_list_props->{$key} = $props->{$key};
    }

    1;
}

sub _make_label_html {
    my ( $prop, $obj ) = @_;
    my $app = MT->instance;

    my $status = $obj->status;
    my $status_class
        = $status == MT::ContentStatus::HOLD()      ? 'Draft'
        : $status == MT::ContentStatus::RELEASE()   ? 'Published'
        : $status == MT::ContentStatus::REVIEW()    ? 'Review'
        : $status == MT::ContentStatus::FUTURE()    ? 'Future'
        : $status == MT::ContentStatus::JUNK()      ? 'Junk'
        : $status == MT::ContentStatus::UNPUBLISH() ? 'Unpublish'
        :                                             '';
    my $lc_status_class = lc $status_class;

    my $status_icon_id
        = $status == MT::ContentStatus::HOLD()      ? 'ic_draft'
        : $status == MT::ContentStatus::RELEASE()   ? 'ic_checkbox'
        : $status == MT::ContentStatus::REVIEW()    ? 'ic_error'
        : $status == MT::ContentStatus::FUTURE()    ? 'ic_clock'
        : $status == MT::ContentStatus::JUNK()      ? 'ic_error'
        : $status == MT::ContentStatus::UNPUBLISH() ? 'ic_stop'
        :                                             '';
    my $status_icon_color_class
        = $status == MT::ContentStatus::HOLD()      ? ''
        : $status == MT::ContentStatus::RELEASE()   ? ' mt-icon--success'
        : $status == MT::ContentStatus::REVIEW()    ? ' mt-icon--warning'
        : $status == MT::ContentStatus::FUTURE()    ? ' mt-icon--info'
        : $status == MT::ContentStatus::JUNK()      ? ' mt-icon--warning'
        : $status == MT::ContentStatus::UNPUBLISH() ? ' mt-icon--danger'
        :                                             '';

    my $status_img = '';
    if ($status_icon_id) {
        my $static_uri = MT->static_path;
        $status_img = qq{
          <svg title="$status_class" role="img" class="mt-icon mt-icon--sm$status_icon_color_class">
              <use xlink:href="${static_uri}images/sprite.svg#$status_icon_id">
          </svg>
        };
    }

    my $label = $obj->label || MT->translate('No Label');
    my $edit_link;
    if ( $app->user->permissions( $obj->blog_id )
        ->can_edit_content_data( $obj, $app->user ) )
    {
        $edit_link = $app->uri(
            mode => 'view',
            args => {
                _type           => 'content_data',
                id              => $obj->id,
                blog_id         => $obj->blog_id,
                content_type_id => $obj->content_type_id,
            },
        );
    }

    my $permalink  = MT::Util::encode_html( $obj->permalink );
    my $static_uri = MT->static_path;
    my $view_link  = $status == MT::ContentStatus::RELEASE()
        ? qq{
            <span class="view-link">
              <a href="$permalink" class="d-inline-block" target="_blank">
                <svg title="View" role="img" class="mt-icon mt-icon--sm">
                  <use xlink:href="${static_uri}images/sprite.svg#ic_permalink">
                </svg>
              </a>
            </span>
        }
        : '';

    if ($edit_link) {
        return qq{
            <span class="icon status $lc_status_class">
              <a href="$edit_link" class="d-inline-block">$status_img</a>
            </span>
            <a href="$edit_link">$label</a>
            $view_link
        };
    }
    else {
        return qq{
            <span class="icon status $lc_status_class">
              $status_img
            </span>
            $label
            $view_link
        };
    }
}


sub out_html_cd {
    my ( $prop, $content_data, $app ) = @_;
#MT->instance->log($prop->{idx_type});
    my $select = MT->translate('Select');
    my $delete = MT->translate('Delete');

    my $label = $content_data->data->{ $prop->content_field_id };
    if ( $label && ref $label eq 'ARRAY' ) {
        my $delimiter = $app->registry('content_field_types')
            ->{ $prop->{idx_type} }{options_delimiter} || ',';
        $label = join "${delimiter} ", @$label;
    }

    $label = '' unless defined $label;
    my $can_double_encode = 1;
    $label = MT::Util::encode_html( $label, $can_double_encode );

#    return qq{<span class="label">$label</span>};
#-----------

#    if ($prop->type ne 'checkbox') {
#        $label = '---' if !$label;
#    } else {
#        $label = '0' if !$label;
#    }
     $label = '---' if !$label;

    my $id = $content_data->id; # == $obj->id

    my $asset_ids;
    my @list_asset_id;

    my $basename = $prop->{content_field_id};

    my $save_changes_label = MT->translate('Save Changes');
    my $save_label = MT->translate('Save');
    my $cancel_label = MT->translate('Cancel');
    my $element;

    if ($prop->{idx_type} eq 'single_line_text' ||
        $prop->{idx_type} eq 'number' ||
        $prop->{idx_type} eq 'url' ||
        $prop->{idx_type} eq 'date_and_time' ||
        $prop->{idx_type} eq 'date_only' ||
        $prop->{idx_type} eq 'time_only' ||
        $prop->{idx_type} eq 'list' ) {
        if ( $prop->{idx_type} eq 'date_only' ) {
            $label =~ s/(\d{8})\d{6}/$1/;
        }
        if ( $prop->{idx_type} eq 'time_only' ) {
            $label =~ s/\d{8}(\d{6})/$1/;
        }
        $element = qq{<input type="text" name="note-$basename-$id" id="note-$basename-$id" class="lfetext" value="} . $label . qq{" />};

    } elsif ($prop->{idx_type} eq 'multi_line_text' ||
             $prop->{idx_type} eq 'embedded_text' ) {
        $element = qq{<textarea name="note-$basename-$id" id="note-$basename-$id" class="lfetextarea">} . $label . qq{</textarea>};

    } elsif ($prop->{idx_type} eq 'checkboxes') {
        my $content_type = MT::ContentType->load( $content_data->content_type_id );
        my $field_data = $content_type->fields;

        my @label;

        for my $field (@{$field_data}) {
            if ( $prop->content_field_id == $field->{id} ) {
                my $options = $field->{options}->{values};
                my @tmp = split( /, /, $label );
                for my $option (@{$options}) {
                    my $flag = 0;
                    for my $value (@tmp) {
                        if ( $option->{value} eq $value ) {
                            $flag = 1;
                        }
                    }
                    if ( $flag ) {
                        $element .= qq{<input type="checkbox" name="note-$basename-$id" value="} . $option->{value} . qq{" checked="checked" class="lfecheckbox" /> } . $option->{label} . '<br />';
                        push @label, $option->{label};
                    } else {
                        $element .= qq{<input type="checkbox" name="note-$basename-$id" value="} . $option->{value} . qq{" class="lfecheckbox" /> } . $option->{label} . '<br />';
                    }
                }
                last;
            }
        }
        $element = qq{<div id="note-$basename-$id">} . $element . qq{</div>};
        $label = join('<br />', @label);
        $label = '---' if !$label;

    } elsif ($prop->{idx_type} eq 'select_box') {
        my $content_type = MT::ContentType->load( $content_data->content_type_id );
        my $field_data = $content_type->fields;

        my @label;
        my $buf;

        my $multiple;
        for my $field (@{$field_data}) {
            if ( $prop->content_field_id == $field->{id} ) {
                my $options = $field->{options}->{values};
                my @tmp = split( /, /, $label );

                $multiple = $field->{options}->{multiple} ? 'multiple':'';
                $buf .= qq{<option value=""></option>} if $multiple ne 'multiple';

                for my $option (@{$options}) {
                    my $flag = 0;
                    for my $value (@tmp) {
                        if ( $option->{value} eq $value ) {
                            $flag = 1;
                        }
                    }

                    if ( $flag ) {
                        $buf .= qq{<option value="} . $option->{value} . qq{" selected="selected">} . $option->{label} . qq{</option>};
                        push @label, $option->{label};
                    } else {
                        $buf .= qq{<option value="} . $option->{value} . qq{">} . $option->{label} . qq{</option>};
                    }
                }
                last;
            }
        }
        $element = qq{<select $multiple name="note-$basename-$id" id="note-$basename-$id" class="lfeselect">} . $buf . qq{</select>};
        $label = join('<br />', @label);
        $label = '---' if !$label;

    } elsif ($prop->{idx_type} eq 'radio_button') {

        my $content_type = MT::ContentType->load( $content_data->content_type_id );
        my $field_data = $content_type->fields;

        my @label;

        for my $field (@{$field_data}) {
            if ( $prop->content_field_id == $field->{id} ) {
                my $options = $field->{options}->{values};
                for my $option (@{$options}) {
                    if ( $option->{value} eq $label ) {
                        $element .= qq{<input type="radio" name="note-$basename-$id" value="} . $option->{value} . qq{" checked="checked" /> } . $option->{label} . '<br />';
                        push @label, $option->{label};
                    } else {
                        $element .= qq{<input type="radio" name="note-$basename-$id" value="} . $option->{value} . qq{" /> } . $option->{label} . '<br />';
                    }
                }
                last;
            }
        }
        $element = qq{<div id="note-$basename-$id">} . $element . qq{</div>};
        $label = join('<br />', @label);
        $label = '---' if !$label;

    } elsif ($prop->{idx_type} eq 'categories') {
        my $content_field = MT::ContentField->load( { id => $prop->content_field_id, blog_id => $app->blog->id, content_type_id => $content_data->content_type_id } );
        my $cat_set_id = $content_field->related_cat_set_id;

        my @categories = MT::Category->load({ blog_id => $app->blog->id, category_set_id => $cat_set_id });

        my $content_type = MT::ContentType->load( $content_data->content_type_id );
        my $field_data = $content_type->fields;

        my @label;
        my $buf;

        my $multiple;
        for my $field (@{$field_data}) {
            if ( $prop->content_field_id == $field->{id} ) {
                $multiple = $field->{options}->{multiple} ? 'multiple':'';

                my @tmp = split( /, /, $label );
#                if ( $multiple ) {
#
#                for my $cat (@categories) {
#                    my $flag = 0;
#                    for my $value (@tmp) {
#                        if ( $cat->id eq $value ) {
#                            $flag = 1;
#                        }
#                    }
#                    if ( $flag ) {
#                        $element .= qq{<input type="checkbox" name="note-$basename-$id" value="} . $cat->id . qq{" checked="checked" class="lfecheckbox" /> } . $cat->label . '<br />';
#                        push @label, $cat->label;
#                    } else {
#                        $element .= qq{<input type="checkbox" name="note-$basename-$id" value="} . $cat->id . qq{" class="lfecheckbox" /> } . $cat->label . '<br />';
#                    }
#                }
#
#                } else {

                $buf .= qq{<option value=""></option>} if $multiple ne 'multiple';
                for my $cat (@categories) {
                    my $flag = 0;
                    for my $value (@tmp) {
                        if ( $cat->id eq $value ) {
                            $flag = 1;
                        }
                    }
                    if ( $flag ) {
                        $buf .= qq{<option value="} . $cat->id . qq{" selected="selected">} . $cat->label . qq{</option>};
                        push @label, $cat->label;
                    } else {
                        $buf .= qq{<option value="} . $cat->id . qq{">} . $cat->label . qq{</option>};
                    }
                }

#                }
                last;
            }
        }
        $element = qq{<select $multiple name="note-$basename-$id" id="note-$basename-$id" class="lfeselect">} . $buf . qq{</select>};
        $label = join('<br />', @label);
        $label = '---' if !$label;

    } elsif ($prop->{idx_type} eq 'tags') {
        my $content_type = MT::ContentType->load( $content_data->content_type_id );
        my $field_data = $content_type->fields;

        my @label;
        my $buf;

        my $can_add;
        my $multiple;
        for my $field (@{$field_data}) {
            if ( $prop->content_field_id == $field->{id} ) {
                my @tmp = split( /, /, $label );

                $can_add = $field->{options}->{can_add} ? 1 : 0;
                $multiple = $field->{options}->{multiple} ? 'multiple':'';

                if ( !$can_add ) {
                    $buf .= qq{<option value=""></option>} if $multiple ne 'multiple';

                    my $terms;
                    my $args;
                    $args->{'join'} ||= MT::ObjectTag->join_on(
                        'tag_id',
                        {   blog_id => $app->blog->id,
                            object_datasource => 'content_data'
                        },{ unique => 1 }
                    );
                    my @tags;
                    my $iter = MT::Tag->load_iter( $terms, $args );

                    while ( my $tag = $iter->() ) {
                        my $flag = 0;
                        for my $id (@tmp) {
                            if ( $id eq $tag->id ) {
                                $flag = 1;
                            }
                        }

                        if ( $flag ) {
                            $buf .= qq{<option value="} . $tag->id . qq{" selected="selected">} . $tag->name . qq{</option>};
                            push @label, $tag->name;
                        } else {
                            $buf .= qq{<option value="} . $tag->id . qq{">} . $tag->name . qq{</option>};
                        }
                    }
                    $element = qq{<select $multiple name="note-$basename-$id" id="note-$basename-$id" class="lfeselect">} . $buf . qq{</select>};
                    $label = join('<br />', @label);
                } else {
                    for my $id (@tmp) {
                        my $tag = MT::Tag->load($id);
                        push @label, $tag->name;
                    }
                    my $tmp_label = join(', ', @label);
                    $element = qq{<input type="text" name="note-$basename-$id" id="note-$basename-$id" class="lfetext" value="} . $tmp_label . qq{" />};
                    $label = join('<br />', @label);
                }
                last;
            }
        }

        $label = '---' if !$label;

    } elsif ($prop->{idx_type} eq 'content_type') {

        my $content_field = MT::ContentField->load( { id => $prop->content_field_id, blog_id => $app->blog->id, content_type_id => $content_data->content_type_id } );
        my $inner_content_type_id = $content_field->related_content_type_id;

        # Recf[^炷ׂẴRecf[^擾
        my $inner_content_type = MT::ContentType->load( $inner_content_type_id );
        my @inner_content_data = MT::ContentData->load( { blog_id => $app->blog->id, content_type_id => $inner_content_type->id } );

        my $content_type = MT::ContentType->load( $content_data->content_type_id );
        my $field_data = $content_type->fields;

        my @label;
        my $buf;

        my $multiple;
        for my $field (@{$field_data}) {
            if ( $prop->content_field_id == $field->{id} ) {
                $multiple = $field->{options}->{multiple} ? 'multiple':'';

                my @tmp = split( /, /, $label );
                $buf .= qq{<option value=""></option>} if $multiple ne 'multiple';
                for my $cd (@inner_content_data) {
                    my $flag = 0;
                    for my $value (@tmp) {
                        if ( $cd->id eq $value ) {
                            $flag = 1;
                        }
                    }
                    if ( $flag ) {
                        $buf .= qq{<option value="} . $cd->id . qq{" selected="selected">} . $cd->label . qq{</option>};
                        push @label, $cd->label;
                    } else {
                        $buf .= qq{<option value="} . $cd->id . qq{">} . $cd->label . qq{</option>};
                    }
                }

                last;
            }
        }
        $element = qq{<select $multiple name="note-$basename-$id" id="note-$basename-$id" class="lfeselect">} . $buf . qq{</select>};
        $label = join('<br />', @label);
        $label = '---' if !$label;

    } elsif ($prop->{idx_type} eq 'asset_image') {

        my $content_field_id = $prop->content_field_id;
        my @asset_ids = split( /, /, $label );

        $label = '';
        my @imgs;
        for my $asset_id (@asset_ids) {
            my $asset = MT->model('asset')->load( { id => $asset_id } );
            next if !$asset;
            my ( $url, $w, $h ) = $asset->thumbnail_url( Height => 45, Width => 45, Square => 1 );
            push @imgs, '<img id="image-' . $basename . '-' . $id . '-child" src="'.$url.'" />';
            push @list_asset_id, $asset->id;
        }
        $label = @imgs ? join(' ', @imgs) : '---';
        $asset_ids = join(',', @list_asset_id);

        my $uri = MT->instance->app_uri;
        my $blog_id = MT->instance->blog->id;

        $element = qq{<div id="image-$basename-$id">$label</div><div class="image-link"><a data-mt-modal-large class="mt-open-dialog mt-modal-open d-inline-block" href="${uri}?__mode=list_asset_for_cd&_type=asset&blog_id=$blog_id&obj_id=$id&no_insert=1&basename=$basename&dialog_view=1&filter=class&filter_val=image&require_type=image&content_field_id=$content_field_id&edit_field=customfield_cfimage&asset_select=1'">} . $select . qq{</a> <a href="#$id" class="clear-image" id="delete-$basename-$id">} . $delete . qq{</a></div>};
    }

    my $out;
    if ($prop->{idx_type} ne 'asset_image') {

        $out = qq{<span id="note-$basename-link-$id"><a href="#$id" class="start-edit" title="$label">}.$label.qq{</a></span><span id="note-$basename-field-$id" style="display: none">} . $element . qq{<br /><span class="buttons"><a href="#$id" type="button" title="$save_changes_label" class="button submit submit-note">$save_label</a><a href="#$id" type="button" class="button cancel-edit">$cancel_label</a></span></span>};

    } else {

        $out = qq{<span id="note-$basename-link-$id"><a href="#$id" class="start-edit" style="text-decoration:none; :hover{opacity:0.7}" title="">}.$label.qq{</a></span><span id="note-$basename-field-$id" style="display: none">} . $element . qq{<input class="include_asset_ids-$id" id="include_asset_ids-$basename-$id" type="hidden" value="$asset_ids" name="include_asset_ids" /><input id="note-$basename-$id" class="customfield_$basename-$id" type="hidden" value="$asset_ids" name="customfield_$basename" /><br /><span class="buttons"><a href="#$id" type="button" title="$save_changes_label" class="button submit-note">$save_label</a><a href="#$id" type="button" class="button cancel-edit">$cancel_label</a></span></span>};

    }

    return $out;
}

sub add_script_cd {
    my ($cb, $app, $param, $tmpl) = @_;
    my $type = $app->param('_type');

    my $blog_id = $app->param('blog_id') || 0;
    return if !$blog_id;

    my $q = $app->param;
    my $content_type_id = $q->param('type');
    $content_type_id =~ s/^content_data_(.*)$/$1/;

    my $content_type = MT::ContentType->load({ blog_id => $app->blog->id, id => $content_type_id });

# Regf[^ł܂킷H
#    my $content_data = MT::ContentData->load({ content_type_id => $content_type_id, blog_id => $app->blog->id });
#    my $data = $content_data->data;

#    my @fields = CustomFields::Field->load({
#        obj_type => ($asset_type) ? $asset_type : $type eq 'member' ? 'author' : $type,
#        blog_id => $app->blog ? [ $app->blog->id, 0 ] : 0,
#    });
#    return if !@fields;

#    $type = 'member' if $type eq 'author';

    my $old_buf;
    my $link_buf;
    my $field_buf;
    my $note_buf;
    my $old_note_buf;
    my $empty_note_buf;
    my $empty_old_note_buf;
    my $recover_buf;
    my $clear_note_buf;
    my $new_note_buf;
    my @cancel_buf;
    my $submit_buf;
    my $render_buf;
my $none_buf;
my $all_none_buf;

    my $all_note_buf;
    my $all_old_note_buf;
    my $all_empty_note_buf;
    my $all_empty_old_note_buf;
    my $all_recover_buf;
    my $all_clear_note_buf;
    my $all_new_note_buf;
    my @all_cancel_buf;
    my $all_submit_buf;
    my $all_render_buf;

    my @basenames;
    my $counter = 0;

    my $image_buf;
    my $old_image_buf;
    my $recover_image_buf;
    my $clear_image_buf;

    my $image_asset_id_buf;
    my $old_image_asset_id_buf;
    my $recover_image_asset_id_buf;
    my $clear_image_asset_id_buf;
    my $asset_ids;

    my $all_image_buf;
    my $all_old_image_buf;
    my $all_recover_image_buf;
    my $all_clear_image_buf;
    my $all_asset_ids;

    my $all_note_buf_new;
    my $all_old_note_buf_new;
    my $all_check_note_buf_new;

    for my $field_data ( @{ $content_type->fields } ) {
        my $type = $field_data->{type};

        next if !( $type eq 'single_line_text' ||
                   $type eq 'multi_line_text' ||
                   $type eq 'number' ||
                   $type eq 'checkboxes' ||
                   $type eq 'select_box' ||
                   $type eq 'radio_button' ||
                   $type eq 'url' ||
                   $type eq 'date_and_time' ||
                   $type eq 'date_only' ||
                   $type eq 'time_only' ||
                   $type eq 'categories' ||
                   $type eq 'tags' ||
                   $type eq 'content_type' ||
                   $type eq 'asset_image' );
        my $bn = $field_data->{id};

        $all_note_buf_new .= "all_note_" . $bn . " = new Array();\n";
        $all_old_note_buf_new .= "all_old_note_" . $bn . " = new Array();\n";
$all_old_note_buf_new .= "all_old_note_" . $bn . "_tmp = new Array();\n";
        $all_check_note_buf_new .= "all_check_note_" . $bn . " = new Array();\n";
$all_check_note_buf_new .= "all_check_note_" . $bn . "_tmp = new Array();\n";
    }

#MT->instance->log("fields:".Dumper($content_type->fields));
    for my $field_data ( @{ $content_type->fields } ) {
        my $type = $field_data->{type};

        next if !( $type eq 'single_line_text' ||
                   $type eq 'multi_line_text' ||
                   $type eq 'number' ||
                   $type eq 'checkboxes' ||
                   $type eq 'select_box' ||
                   $type eq 'radio_button' ||
                   $type eq 'url' ||
                   $type eq 'date_and_time' ||
                   $type eq 'date_only' ||
                   $type eq 'time_only' ||
                   $type eq 'categories' ||
                   $type eq 'tags' ||
                   $type eq 'content_type' ||
                   $type eq 'asset_image' );

        my $bn = $field_data->{id};
        $old_buf .=  "var old_note_" . $bn . " = '';\nvar check_note_" . $bn . " = '';\n";

if ( $type eq 'asset_image' ) {
        $old_buf .=  "var old_image_asset_id_" . $bn . " = '';\n";
}

        $link_buf .=  "note_links.push( jQuery('#note-" . $bn . "-link-' + id) );\n";
        $field_buf .= "note_fields.push( jQuery('#note-" . $bn . "-field-' + id) );\n";

        # note
        if ($type eq 'radio_button') {
            $note_buf .= "var note_" . $bn . " = jQuery('[name=\"note-" . $bn . "-' + id + '\"]:checked');\n";
            $all_note_buf .= "all_note_" . $bn . "[id] = jQuery('[name=\"note-" . $bn . "-' + id + '\"]:checked');\n";
        } else {
            $note_buf .= "var note_" . $bn . " = jQuery('#note-" . $bn . "-' + id);\n";
            $all_note_buf .= "all_note_" . $bn . "[id] = jQuery('#note-" . $bn . "-' + id);\n";
        }

        # old
        if ($type eq 'checkboxes') {
            $old_note_buf .= "old_note_" . $bn . ";\nvar old_note_" . $bn ."_tmp = [];\njQuery('input[name=note-" . $bn . "-' + id + ']:checked').each(function(){\n" . "old_note_" . $bn . "_tmp.push(\$(this).val());\n});\nold_note_" . $bn . " = old_note_" . $bn . "_tmp.join(',');\ncheck_note_" . $bn . " = old_note_" . $bn . "_tmp.join(',');\n";
            $all_old_note_buf .= "//all_old_note_" . $bn . "[id];\n//all_old_note_" . $bn ."_tmp[id] = [];\njQuery('input[name=note-" . $bn . "-' + id + ']:checked').each(function(){\n" . "  if (!all_old_note_" . $bn . "_tmp[id]) {\n    all_old_note_" . $bn . "_tmp[id] = new Array();\n  }\n  all_old_note_" . $bn . "_tmp[id].push(\$(this).val());\n});\nif(all_old_note_" . $bn . "_tmp[id]){\n  all_old_note_" . $bn . "[id] = all_old_note_" . $bn . "_tmp[id].join(',');\n  all_check_note_" . $bn . "[id] = all_old_note_" . $bn . "_tmp[id].join(',');\n}\n";
        } elsif ($type eq 'select_box' || $type eq 'categories' || $type eq 'content_type') {
            $old_note_buf .= "old_note_" . $bn . ";\nvar old_note_" . $bn ."_tmp = [];\njQuery('select[name=note-" . $bn . "-' + id + '] option:selected').each(function(){\n" . "old_note_" . $bn . "_tmp.push(\$(this).val());\n});\nold_note_" . $bn . " = old_note_" . $bn . "_tmp.join(',');\ncheck_note_" . $bn . " = old_note_" . $bn . "_tmp.join(',');\n";
            $all_old_note_buf .= "//all_old_note_" . $bn . "[id];\n//all_old_note_" . $bn ."_tmp[id] = [];\njQuery('select[name=note-" . $bn . "-' + id + '] option:selected').each(function(){\n" . "  if (!all_old_note_" . $bn . "_tmp[id]) {\n    all_old_note_" . $bn . "_tmp[id] = new Array();\n  }\n  all_old_note_" . $bn . "_tmp[id].push(\$(this).val());\n});\nif(all_old_note_" . $bn . "_tmp[id]){\n  all_old_note_" . $bn . "[id] = all_old_note_" . $bn . "_tmp[id].join(',');\n  all_check_note_" . $bn . "[id] = all_old_note_" . $bn . "_tmp[id].join(',');\n}\n";
        } else {
            $old_note_buf .= "old_note_" . $bn . " = note_" . $bn. ".val();\ncheck_note_" . $bn . " = note_" . $bn . ".val();\n";
            $all_old_note_buf .= "all_old_note_" . $bn . "[id] = all_note_" . $bn . "[id].val();\nall_check_note_" . $bn . "[id] = all_note_" . $bn . "[id].val();\n";
        }

        # empty
        if ($type eq 'checkboxes') {
            $empty_note_buf .= "var check_note_" . $bn . "_tmp = [];\njQuery('input[name=note-" . $bn . "-' + id + ']:checked').each(function(){\n" . "check_note_" . $bn . "_tmp.push(\$(this).val());\n});\nif ( !check_note_" . $bn . "_tmp.length ) {\ncheck_note_" . $bn . " = '';\n}\n";
            $all_empty_note_buf .= "if (jQuery('#note-" . $bn . "-' + id).attr('checked') == 'checked' || jQuery('#note-" . $bn . "-' + id).attr('checked') == true) {\n  all_note_" . $bn . "[id] = 1;\n  all_check_note_" . $bn . "[id] = 1;\n} else {\n  all_note_" . $bn . "[id] = 0;\n  all_check_note_" . $bn . "[id] = 0;\n}\n";
        } elsif ($type eq 'select_box' || $type eq 'categories' || $type eq 'content_type') {
            $empty_note_buf .= "var check_note_" . $bn . "_tmp = [];\njQuery('select[name=note-" . $bn . "-' + id + '] option:selected').each(function(){\n" . "check_note_" . $bn . "_tmp.push(\$(this).val());\n});\nif ( !check_note_" . $bn . "_tmp.length ) {\ncheck_note_" . $bn . " = '';\n}\n";
            $all_empty_note_buf .= "//check_note_" . $bn . "_tmp[id] = [];\njQuery('select[name=note-" . $bn . "-' + id + '] option:selected').each(function(){\n  all_check_note_" . $bn . "_tmp.push(\$(this).val());\n});\nif ( !all_check_note_" . $bn . "_tmp.length ) {\n  all_check_note_" . $bn . "[id] = '';\n}\n";
        } else {
            $empty_note_buf .= "if (note_" . $bn . ".val() == '---'){\n  note_" . $bn . ".val('');\n  check_note_" . $bn . " = '';\n}\n";
            $all_empty_note_buf .= "if (all_note_" . $bn . "[id].val() == '---'){\n  all_note_" . $bn . "[id].val('');\n  all_check_note_" . $bn . "[id] = '';\n}\n";
        }

        # recover
        if ($type ne 'checkboxes') {
            $recover_buf .= "note_" . $bn . ".val( old_note_" . $bn . " );\n";
            $all_recover_buf .= "all_note_" . $bn . "[id].val( all_old_note_" . $bn . "[id] );\n";
        } else {
            $recover_buf .= "note_" . $bn . ".attr('checked', old_note_" . $bn . ");\n";
            $all_recover_buf .= "all_note_" . $bn . "[id].val( all_old_note_" . $bn . "[id] );\n";
        }
        $clear_note_buf .=  "old_note_" . $bn . " = '';\n";
        $all_clear_note_buf .=  "all_old_note_" . $bn . "[id] = '';\n";

        # new
        if ($type eq 'checkboxes') {
            $new_note_buf .= "var new_note_" . $bn . ";\nvar new_note_" . $bn ."_tmp = [];\njQuery('input[name=note-" . $bn . "-' + id + ']:checked').each(function(){\n" . "new_note_" . $bn . "_tmp.push(\$(this).val());\n});\nnew_note_" . $bn . " = new_note_" . $bn . "_tmp.join(',');\n";
            $all_new_note_buf .= "var all_new_note_" . $bn . "[id];\nall_new_note_" . $bn ."_tmp[id] = [];\njQuery('input[name=note-" . $bn . "-' + id + ']:checked').each(function(){\n" . "all_new_note_" . $bn . "_tmp[id].push(\$(this).val());\n});\nall_new_note_" . $bn . "[id] = all_new_note_" . $bn . "_tmp[id].join(',');\n";
        } elsif ($type eq 'select_box' || $type eq 'categories' || $type eq 'content_type') {
            $new_note_buf .= "var new_note_" . $bn . ";\nvar new_note_" . $bn ."_tmp = [];\njQuery('select[name=note-" . $bn . "-' + id + '] option:selected').each(function(){\n" . "new_note_" . $bn . "_tmp.push(\$(this).val());\n});\nnew_note_" . $bn . " = new_note_" . $bn . "_tmp.join(',');\n";
            $all_new_note_buf .= "var all_new_note_" . $bn . "[id];\nnew_note_" . $bn ."_tmp[id] = [];\njQuery('select[name=note-" . $bn . "-' + id + '] option:selected').each(function(){\n" . "all_new_note_" . $bn . "_tmp[id].push(\$(this).val());\n});\nall_new_note_" . $bn . "[id] = all_new_note_" . $bn . "_tmp[id].join(',');\n";
        } else {
            $new_note_buf .= "var new_note_" . $bn . " = note_" . $bn . ".val();\n";
            $all_new_note_buf .= "var all_new_note_" . $bn . "[id] = all_note_" . $bn . "[id].val();\n";
        }

        push @cancel_buf, "check_note_" . $bn . " == new_note_" . $bn;
        push @all_cancel_buf, "all_check_note_" . $bn . "[id] == all_new_note_" . $bn . "[id]";

        # submit
        if ($type eq 'radio_button') {
            $submit_buf .= "var note_" . $bn . " = jQuery('[name=\"note-" . $bn . "-' + id + '\"]:checked').val();\n";
            $all_submit_buf .= "var note_" . $bn . " = jQuery('[name=\"note-" . $bn . "-' + id + '\"]:checked').val();\n";
        } elsif ($type eq 'checkboxes') {
            $submit_buf .= "var note_" . $bn ."_tmp = [];\njQuery('input[name=note-" . $bn . "-' + id + ']:checked').each(function(){\n" . "note_" . $bn . "_tmp.push(\$(this).val());\n});\nvar note_" . $bn . " = note_" . $bn . "_tmp.join(',');\n";
            $all_submit_buf .= "var note_" . $bn ."_tmp = [];\njQuery('input[name=note-" . $bn . "-' + id + ']:checked').each(function(){\n" . "note_" . $bn . "_tmp.push(\$(this).val());\n});\nvar note_" . $bn . " = note_" . $bn . "_tmp.join(',');\n";
        } elsif ($type eq 'select_box' || $type eq 'categories' || $type eq 'content_type') {
            $submit_buf .= "var note_" . $bn ."_tmp = [];\njQuery('select[name=note-" . $bn . "-' + id + '] option:selected').each(function(){\n" . "note_" . $bn . "_tmp.push(\$(this).val());\n});\nvar note_" . $bn . " = note_" . $bn . "_tmp.join(',');\n";
            $all_submit_buf .= "var note_" . $bn ."_tmp = [];\njQuery('select[name=note-" . $bn . "-' + id + '] option:selected').each(function(){\n" . "note_" . $bn . "_tmp.push(\$(this).val());\n});\nvar note_" . $bn . " = note_" . $bn . "_tmp.join(',');\n";
        } else {
            $submit_buf .= "var note_" . $bn . " = jQuery('#note-" . $bn . "-' + id).val();\n";
            $all_submit_buf .= "var note_" . $bn . " = jQuery('#note-" . $bn . "-' + id).val();\n";
        }

        # render
        $render_buf .= "customfield_" . $bn . ": note_" . $bn . ",\n";
        $all_render_buf .= "param['customfield_" . $bn . "'] =  note_" . $bn . ";\n";

        $none_buf .= "if ( jQuery('#note-" . $bn . "-' + id)[0] ) {} else { note_" . $bn . " = 'none'; }\n";
        $all_none_buf .= "if ( jQuery('#note-" . $bn . "-' + id)[0] ) {} else { note_" . $bn . " = 'none'; }\n";

        # asset_ids
        $asset_ids = "var asset_ids = new Array();\nvar asset_ids_val = '';\njQuery('.include_asset_ids-' + id).each(function(){\nasset_ids[asset_ids.length] = jQuery(this).val();\n});\n
if(asset_ids.length > 1){\nasset_ids_val = '[' + asset_ids.join(',') + ']';\n} else if(asset_ids.length == 1){\nasset_ids_val = asset_ids[0];\n} else {\nasset_ids_val = 0;\n}\n";

        if ($type eq 'asset_image') {
            $old_image_buf .= "var old_image_" . $bn . " = '';\n";
            $image_buf .= "old_image_" . $bn . " = jQuery('#image-" . $bn . "-' + id).html();\n";
            $recover_image_buf .= "if(old_note_" . $bn . " == '---') {\nold_note_" . $bn . " = '';\n}\njQuery('#image-" . $bn . "-' + id).html( old_image_" . $bn . " );\nif (old_image_" . $bn . " == '---'){\n  jQuery('#delete-" . $bn . "-' + id).css('display', 'none');\n} else {\n  jQuery('#delete-" . $bn . "-' + id).css('display', 'inline');\n}\n";
            $clear_image_buf .=  "old_image_" . $bn . " = '';\n";

            $image_asset_id_buf .= "old_image_asset_id_" . $bn . " = jQuery('#include_asset_ids-" . $bn . "-' + id).val();\n";
            $recover_image_asset_id_buf .= "jQuery('#include_asset_ids-" . $bn . "-' + id).val( old_image_asset_id_" . $bn . " );\n";
            $clear_image_asset_id_buf .=  "old_image_asset_id_" . $bn . " = '';\n";
        }

        $counter++;
    }
    return 1 if !$counter;
    my $cancel = join(' && ', @cancel_buf);

    my $plugin = $app->component('PowerListingFieldEditor');
    my $field = $tmpl->getElementById('page-title');
    my $node = $tmpl->createElement('setvarblock', { name => 'jq_js_include', component => $plugin });

    my $content_data_id_message = $plugin->translate('content_data_id');
    my $update_message = $plugin->translate('update...');
    my $end_message = $plugin->translate('done update');

    my $html = <<HTML;
    var current_edit = 0;
    $old_buf
    $old_image_buf

    function clear_image(id, basename_id) {
        basename_id.match(/delete-(.*)/);
        var basename_id = RegExp.\$1;
        jQuery('#image-' + basename_id).html('---');
        jQuery('#note-' + basename_id).val('');
        jQuery('#include_asset_ids-' + basename_id).val('');
        jQuery('#delete-' + basename_id).css('display', 'none');
    }

    function edit_note(id) {
        if (current_edit) return;
        var note_links = new Array();
        $link_buf
        var note_fields = new Array();
        $field_buf
        if (!note_links.length || !note_fields.length) return;
        $note_buf
        current_edit = id;
        $old_note_buf
        $image_buf
        $image_asset_id_buf
        $empty_note_buf

        for ( i = 0; i < $counter; ++i ) {
            note_links[i].hide();
            note_fields[i].show();
        }
        return false;
    }

    function cancel_edit(id) {
        var note_links = new Array();
        $link_buf
        var note_fields = new Array();
        $field_buf
        $note_buf
        if (!note_links.length || !note_fields.length) return;
        $recover_image_buf
        $recover_image_asset_id_buf
        $recover_buf
        $clear_image_buf
        $clear_image_asset_id_buf
        $clear_note_buf
        current_edit = 0;

        for ( i = 0; i < $counter; ++i ) {
            note_links[i].show();
            note_fields[i].hide();
        }
        return false;
    }

    function submit_edit(id) {
        if ((0 != current_edit) && (id != current_edit))
            id = current_edit;
        $note_buf
        $new_note_buf

        if ($cancel) {
            cancel_edit(id);
            return false;
        }
        return submit_edit_form(id);
    }

    function submit_edit_form(id) {
        $submit_buf
        $asset_ids

$none_buf

        renderList({
            __mode: 'save_from_list_cd',
            _type: '$type',
//            customfield_beacon: 1,
            $render_buf
//            include_asset_ids: asset_ids_val,
            id: id,
            forward_list: 1,
            magic_token: 1
        });
        cancelCreateMode();
        return false;
    }

    function setCreateMode () {
        jQuery('#create-inline-notification').show();
        jQuery('#text').val('').focus();
        jQuery('#content-actions').hide();
    }

    function cancelCreateMode () {
        jQuery('#create-inline-notification').hide();
        jQuery('#create-inline-notification input').mtUnvalidate();
        jQuery('#content-actions').show();

        current_edit = 0;
    }

    jQuery('.start-edit').live('click', function () {
        jQuery(this).attr('href').match(/#(\\d+)/);
        var id = RegExp.\$1;
        cancel_edit(current_edit);
        edit_note(id);

        // add css for selected.
        \$("select option:selected").each(function(){
            \$(this).css('background-color', '#ccc!important');
        });

// [__CAO\̂߂̑Ώ
\$('.mt-modal-open').mtModal();
\$('[data-mt-modal-close]').mtModalClose();
        return false;
    });

    jQuery('.cancel-edit').live('click', function () {
        jQuery(this).attr('href').match(/#(\\d+)/);
        var id = RegExp.\$1;
        cancel_edit(id);
        return false;
    });

    jQuery('.submit-note').live('click', function () {
        jQuery(this).attr('href').match(/#(\\d+)/);
        var id = RegExp.\$1;
        submit_edit(id);
        return false;
    });

    jQuery(window).bind('listReady', function() {
//        jQuery('a.mt-open-dialog').mtDialog();

        jQuery('.clear-image').each(function () {
            jQuery(this).attr('href').match(/#(\\d+)/);
            var id = RegExp.\$1;
            jQuery(this).attr('id').match(/delete-(.*)/);
            var basename_id = RegExp.\$1;
            if (jQuery('#image-' + basename_id).html() == '---'){
                jQuery(this).css('display', 'none');
            } else {
                jQuery(this).css('display', 'inline');
            }
        });

//    jQuery('input[name=id]').live('click', function () {
//        if ( jQuery(this).attr('id').match(/select_(\\d+)/) ) {
//            var id = RegExp.\$1;
//            jQuery(this).attr('checked', 'checked');
//            jQuery(this).parent().parent().parent().attr('checked', 'checked');
//            jQuery(this).parent().parent().parent().attr('class', 'mt-table__highlight');
//        }
//        return false;
//    });

//////////////////////
// ҏWATvtB[hBȂ
// ҏWA`FbN{bNX`FbNłȂ
    jQuery('input[type=checkbox] .custom-control-input.excerpt').live('change', function () {
//alert("OK");
    });
//    jQuery('.excerpt description').display('none');
//////////////////////


    });

    jQuery('.clear-image').live('click', function () {
        jQuery(this).attr('href').match(/#(\\d+)/);
        var id = RegExp.\$1;
        var basename = jQuery(this).attr('id');
        clear_image(id, basename);
        return false;
    });

    // delete css for unselected.
    jQuery('select').live('change',function(){
        \$(this).parent().find('option').each(function(){
            if ( \$(this).prop("selected") ){
            } else {
                \$(this).css('background-color', '#fff!important');
            }
        });
    });

    jQuery('#uploadStatePnl').hide();
    jQuery(window).bind('listReady', function() {

// remove checkbox
jQuery('.col.head.cb.first-visible-child.first-child input').removeAttr('checked');
jQuery('#content_data-table tr').removeClass('selected');
jQuery('#page-table tr').removeClass('selected');

        // end save
        if (done) {
            incr();
            updateStatus(last_id + 1);
            done = 0;
jQuery('#create-inline-notification').hide();
jQuery('#create-inline-notification input').mtUnvalidate();

            jQuery('#uploadStatePnl').animate({
                opacity: 'toggle',}, { duration: 2000, easing: 'swing',
                complete: function () {
                    jQuery("#content-actions").show();
                    jQuery("#display-options").show();

        jQuery("#menu").show();
        jQuery(".has-menu-nav #content").css('margin-left', '175px');
        jQuery("#sidebar_none a").show();
        jQuery("#sidebar_block a").hide();

                }
            });
        // 一括編雁EにフィールドE増渁E
        } else {
            if(jQuery('#create-inline-notification').css('display') == 'none') {
                jQuery("#content-actions").show();
            }
            if(jQuery('#create-inline-notification').css('display') == 'block') {
// tH[f[^ؑ֎ɔ΂邽߃RgAEg
//                setEditAllMode();
            }
        }
    });

    jQuery("#progressbar").progressbar();
    jQuery('#submit-all-field').click( function() {
        submit_all_edit();
        return false;
    });

    jQuery('#action-edit-all').click( function() {
        jQuery("#content-actions").hide();
//        jQuery("#display-options").hide();

// [__CAO\̂߂̑Ώ
\$('.mt-modal-open').mtModal();
\$('[data-mt-modal-close]').mtModalClose();

        setEditAllMode();
        return false;
    });

    jQuery('#create-inline-notification button.mt-cancel-action').click(function() {
//        jQuery("#display-options").show();
        cancelEditAllMode();
        return false;
    });

    jQuery('#sidebar_none a').click(function(){
        jQuery(".mt-primaryNavigation").hide();

        jQuery("#sidebar_none a").hide();
        jQuery("#sidebar_block a").show();
    });
    jQuery('#sidebar_block a').click(function(){
        jQuery(".mt-primaryNavigation").show();

        jQuery("#sidebar_none a").show();
        jQuery("#sidebar_block a").hide();
    });

    function cancelEditAllMode () {
        jQuery('#content_data-table input[type=checkbox]').each(function() {
var id = this.id;
id.match(/select_(\\d+)/);
var data = RegExp.\$1;
            cancel_edit_all(data);

            jQuery('.buttons', jQuery(this).parent().parent().parent()).each(function() {
                jQuery(this).show();
            });
        });
        jQuery('#create-inline-notification').hide();
        jQuery('#content-actions').show();

        jQuery("#menu").show();
        jQuery(".has-menu-nav #content").css('margin-left', '175px');
        jQuery("#sidebar_none a").show();
        jQuery("#sidebar_block a").hide();

        current_edit = 0;
    }

    function setEditAllMode () {
        jQuery('#create-inline-notification').show();
        jQuery('#content-actions').hide();

        cancel_edit(current_edit);

        $all_note_buf_new
        $all_old_note_buf_new
        $all_check_note_buf_new

        jQuery('#content_data-table input[type=checkbox]').each(function() {
var id = this.id;
id.match(/select_(\\d+)/);
var data = RegExp.\$1;
if ( data ) {
            edit_all_note(data);

// add css for selected.
//\$("select option:selected").each(function(){
//    \$(this).css('background-color', '#ccc!important');
//});

            jQuery('.buttons', jQuery(this).parent().parent().parent()).each(function() {
                jQuery(this).hide();
            });
}
        });
        current_edit = 'all';
    }

    function edit_all_note(id) {
        var note_links = new Array();
        $link_buf
        var note_fields = new Array();
        $field_buf
        if (!note_links.length || !note_fields.length) return;
        $all_note_buf
        $all_old_note_buf
        $all_image_buf
        $all_empty_note_buf

        for ( i = 0; i < $counter; ++i ) {
            note_links[i].hide();
            note_fields[i].show();
        }
        return false;
    }

    function cancel_edit_all(id) {
        var note_links = new Array();
        $link_buf
        var note_fields = new Array();
        $field_buf
        $all_note_buf
        if (!note_links.length || !note_fields.length) return;
//        $all_recover_image_buf
//        $all_recover_buf
//        $all_clear_image_buf
//        $all_clear_note_buf
        current_edit = 0;

        for ( i = 0; i < $counter; ++i ) {
            note_links[i].show();
            note_fields[i].hide();
        }
        return false;
    }

    var PageState = {
        SELECT: 0,
        UPLOADING: 1
    };
    var total;
    var done = 0;
    var beginTime;
    var incr = function(){
        ++done;
    };
    var isAbort = false;
    var abort = function(){
        isAbort = true;
        updatePage(PageState.SELECT);
        alert("stopped!");
    };
    var updatePage = function(mode){
        switch(mode){
            case PageState.SELECT:
                jQuery("#content-actions").show();
                jQuery("#uploadStatePnl").hide();
                jQuery("#display-options").show();
                break;
            case PageState.UPLOADING:
                jQuery("#display-options").hide();
                jQuery("#content-actions").hide();
                jQuery("#filePreviewPnl").hide();
                jQuery("#uploadStatePnl").show();
                updateProgress(0);
                break;
        }
    }
    var updateProgress = function(value){
        jQuery("#progressbar").progressbar("value", value);
    }
    var updateStatus = function(id){
        updateProgress(done / total * 100);

        var exec_file = id;
        if (done < total) {
            exec_file = id;
            jQuery("#progressmsg").text("$content_data_id_message" + id + '$update_message' + " ( " + done + " / " + total + " $end_message )");
        } else if (done == total) {
            jQuery("#progressmsg").text(" $end_message(" + done + " / " + total + ")");
        }
    };

    var last_id = 0;
    var submit_all_edit = function() {
        var i = 0;
        var list = new Array();
        jQuery('#content_data-table input[type=checkbox]').each(function() {
            var id = this.id;
            if (id.match(/select_(\\d+)/)) {
                var data = RegExp.\$1;
                list.push(data);
            }
        });

        var len = list.length;
        total = list.length;
        var retry = 0;
        var iter = function(){
                var id = list[i++];
                $all_submit_buf
                $asset_ids

$all_none_buf

                var param = {};
                param['__mode'] = 'save_from_list_cd';
                param['_type'] = 'content_data';
                param['blog_id'] = $blog_id;
                $all_render_buf
                param['id'] = id;
                param['forward_list'] = 1;
                param['magic_token'] = listStore.listClient.magicToken;
                param['content_type_id'] = $content_type_id;
                param['datasource'] = '<mt:var name="list_type">';
                jQuery.ajax({
                    url: script_url,
                    type: "POST",
                    data: param,
                    success: function() {
                        if (isAbort) {
                            return;
                        }
                        if (i < len) {
                            incr();
                            updateStatus(list[i]);
                            retry = 0;
                            iter();
                        } else {
                            incr();
                            updateStatus(id);
                            listStore.trigger('load_list');
                        }
                    },
                    error: function(xhr, textStatus, errorThrown) {
                        if (isAbort) {
                            return;
                        }
                        console.log(xhr);
                        console.log(textStatus + ": " + errorThrown);

                        if (xhr.status == 500 && retry < 3) {
                            console.log("retry: " + this.id);

                            i--;
                            retry++;
                            iter();
                        } else {
                            alert( this.id + "is error");
                            if(confirm("continue?")){
                                iter();
                            } else {
                                abort();
                            }
                        }
                    }
                });
        };

        updateStatus(list[i]);

        jQuery('.indicator, #listing-table-overlay').show();
        jQuery('div.pagination').hide();
        jQuery('#msg-block div.msg-error.filtered-list-msg').remove();
jQuery('#create-inline-notification').hide();
        updatePage(PageState.UPLOADING);
        iter();

//        cancelCreateMode();
        return false;
    };

var cols = [];
var vals = [];
var checked = [];
var total = 0;
var editable_total = 0;
var editable_count_in_page;
var currentPage = 1;

var request_id = 0;
function renderList(mode, columns, values, limit, page ) {
  var this_request = ++request_id;

var params = [];
jQuery('.custom-control.custom-checkbox input[type=checkbox]:checked').each(function(){
    if (this.id.match(/select/)) {
    } else {
    params.push(this.id);
    }
});

  // If the first arg is Object, it's Hash style call
  var args = {};
  if ( typeof mode == 'object' ) {
    args = mode;
    var defaults = {
      mode: 'filtered_list',
      columns: listStore.columns.join(','),
      values:  vals,
      limit: listStore.limit,
      page: listStore.page
    };
    mode    = ( 'mode' in args )    ? args['mode']    : defaults['mode'];
    columns = listStore.columns.join(',');
    values  = ( 'values' in args )  ? args['values']  : defaults['values'];
    limit   = listStore.limit;
    page    = listStore.page;
    delete args['mode'];
    delete args['columns'];
    delete args['values'];
    delete args['limit'];
    delete args['page'];
  }
  limit = parseInt(limit);
  if ( args['magic_token'] ) {
    args['magic_token'] = jQuery('input[name="magic_token"]').val();
  }
  var params = {
    __mode: mode,
    datasource: '<mt:var name="list_type">',
    blog_id: <mt:var name="blog_id">,
    content_type_id: $content_type_id,
    columns: params.join(','),
    limit: listStore.limit,
    page: listStore.page,
    magic_token: listStore.listClient.magicToken,
//items: [],
sort_by: 'authored_on',
sort_order: 'descend',
fid: '_allpass'

  };
  params = jQuery.extend( params, args );

  if (values.length) {
    params['items'] = jQuery.toJSON(values);
  }
  if (jQuery('table.mt-table th.sorted').length) {
    var sorted = jQuery('table.mt-table th.sorted').attr('data-id');
    params['sort_by'] = sorted;
    params['sort_order'] = jQuery('table.mt-table th.sorted').find('span').hasClass('desc') ? 'descend' : 'ascend';
  }
  if (jQuery('input[name=filter_id]').val()) {
    params['fid'] = jQuery('input[name=filter_id]').val();
  }
  if (jQuery('input[name=filter_name]').val()) {
    params['label'] = jQuery('input[name=filter_name]').val();
  }

  jQuery('.indicator, #listing-table-overlay').show();
  jQuery('div.pagination').hide();
  jQuery('#msg-block div.msg-error.filtered-list-msg').remove()
  // adding random bits to avoid cache in Android.
  var script_url = '<mt:var name="script_url">?.r=' + Math.random(1);
  jQuery.ajax({
    type: 'POST',
    contentType: 'application/x-www-form-urlencoded; charset=utf-8',
    url: script_url,
    dataType: 'json',
    data: params,
    success: function(data) {
      if ( this_request != request_id ) {
          return false;
      }

      if ( !handleMessages(data, 'filtered-list-msg' ) ) return;
<mt:if name="config.debugmode">
      if (data.result.debug) {
        jQuery('#listing-debug-block').text(data.result.debug);
      }
</mt:if>
      jQuery('table.mt-table tbody').empty();
      jQuery('table.listing-table th :checkbox').prop('checked',false);
      var objs = data.result.objects;
      var count = objs.length;
//console.log("count:"+count);
      var last = data.result.page_max;
      total = parseInt(data.result.count);
      editable_total = parseInt(data.result.editable_count);
      var return_columns = data.result.columns.split(',');
//console.log("return_columns:"+return_columns);
      var return_cols = jQuery.grep( return_columns, function( val, idx ){
        return val.match(/\./);
      }, true );
      var main_cols = {};
      jQuery('#disp_cols input.main.checkbox:checked').each(function() {
        main_cols[ jQuery(this).val() ] = 1;
      });
      editable_count_in_page = 0;
      for (var row = 0; row < count; row++) {
//console.log("row:"+row);
        var id = objs[row][0];
        var line = '';
        if ( id ) {
          editable_count_in_page++;
          if (checked[page] && checked[page][id]) {
//            line = '<td class="cb col"<mt:unless name="has_list_actions"> style="display: none;"</mt:unless>><input type="checkbox" checked="checked" name="id" value="'+id+'" /></td>';
              line = '<td><div class="custom-control custom-checkbox"><input type="checkbox" name="id" class="custom-control-input" id="select_' + (row+1) + ' value="' + id + '"><span class="custom-control-indicator"></span><label class="custom-control-label" for="select_' + (row+1) + '"><span class="sr-only"><__trans phrase="Select" escape="js"></span></label></div></td>';
          } else {
//            line = '<td class="cb col"<mt:unless name="has_list_actions"> style="display: none;"</mt:unless>><input type="checkbox" name="id" value="'+id+'" /></td>';
              line = '<td><div class="custom-control custom-checkbox"><input type="checkbox" name="id" class="custom-control-input" id="select_' + (row+1) + ' value="' + id + '"><span class="custom-control-indicator"></span><label class="custom-control-label" for="select_' + (row+1) + '"><span class="sr-only"><__trans phrase="Select" escape="js"></span></label></div></td>';
          }
        }
        else {
          line = '<td class="cb col"<mt:unless name="has_list_actions"> style="display: none;"</mt:unless>></td>';
        }
        for (var col = 1, col_length = objs[row].length; col < col_length; col++) {
//console.log("col:"+col+ " data:"+objs[row][col]);
          var col_id = return_cols[ col - 1 ];
//          if ( main_cols[col_id] )
//              line += '<td class="col '+col_id+' '+'">' + objs[row][col] + '</td>';
// contentɒlƃCAEgꂪ
              line += '<td data-is="list-table-column" class="' + col_id + '" content="' + '' + '">' + objs[row][col] + '</td>';
        }
        jQuery('<tr data-is="list-table-row" data-index="' + row + '"' + ( id ? '' : ' class="no-action"' ) + '>'+line+'</tr>').appendTo('table.mt-table tbody');
      }
      jQuery('#resultstats').remove();
      var n = columnLength();
      if (count == 0) {
        var message = trans('No [_1] could be found.', '<mt:if name="zero_state"><mt:var name="zero_state" escape="js"><mt:else><mt:var name="object_label" lower_case="1" escape="js"></mt:if>');
        jQuery('<td class="col fullwidth-row" colspan="'+n+'">'+message+'</tr>').appendTo('table.mt-table tbody');
      }
//      renderPagination(count, limit, page, last, total);
      filters = data.result.filters;
      if ( data.result.label ) {
        jQuery('#opener').html(data.result.label);
        jQuery('#filter_name').val(data.result.label);
      }
      if ( data.result.id ) {
        jQuery('#filter_id').val(data.result.id);
        current_id = data.result.id;
      }
    },
    complete: function() {

//listStore.resetColumns();
//          riot
//          self.updateIsLoading(false);
//          if (refreshCurrentFilter) {
//            self.trigger('refresh_current_filter')
//          }
//          self.trigger('refresh_view', { moveToPagination: moveToPagination });

//      if ( this_request != request_id ) {
//          return false;
//      }
//      jQuery('.indicator, #listing-table-overlay').hide();
//      jQuery('div.pagination').show();

//      jQuery('table.listing-table tbody tr:even').addClass('even');
//      jQuery('table.listing-table tbody tr:odd').addClass('odd');
//      var \$checkboxes = jQuery('td :checkbox');
//      var \$checked = jQuery('td :checked');
//      \$checked.parents('tr').addClass('selected');
//      if (\$checkboxes.length && \$checkboxes.length == \$checked.length) {
//        jQuery('table.listing-table th :checkbox').prop('checked', true);
//        addSelectAll(\$checkboxes);
//      }
//      if (mode == 'save_filter' || mode == 'delete_filter') {
//        renderFilterList();
//        updateFilterSaveButtons({
//          system:     0,
//          new_filter: 0
//        });
//      }

//      if (!jQuery('#msg-block .msg-error').length) {
//        jQuery('#dialog_save_filter').dialog('close');
//      } else {
//        if (jQuery('#dialog_save_filter:visible').length) {
//          jQuery('.msg-error').clone().appendTo('#dialog-msg-block');
//          jQuery('#msg-block').hide();
//        }
//      }
//      jQuery(window).trigger('listReady');



    },
    error: function(xhr, status, error) {
      if ( this_request != request_id ) {
          return false;
      }
      if ( xhr.status == 401 ) {
        loginAgain(function(){
          renderList(mode, columns, values, limit, page);
        });
      }
      else if ( xhr.status == 0 ) {
        // Maybe this is user abort. do nothing.
      }
      else {
        alert( trans('Communication Error (HTTP status code: [_1]. Message: [_2])', xhr.status, error) );
      }
    }
  });
}

function columnLength () {
  return jQuery('#disp_cols').find('input.main:checked').length<mt:if name="has_list_actions"> + 1</mt:if>;
}

function renderPagination(count, limit, page, last, total) {
  var \$e = jQuery('.pagination').empty();
  if (count == 0) {
    return;
  }
  var usefirst = false;
  var uselast = false;
  var start;
  if (page > 1) {
    usefirst = true;
    start = (page-1)*limit+1;
  } else {
    start = 1;
  }
  if (total / limit > 1) {
    if (page + 1 <= last) {
      uselast = true;
    }
  }
  currentPage = page;

  if (usefirst) {
    \$e.append('<a href="#" class="pagenav start">&laquo; <__trans phrase="First" escape="js"></a>')
      .append('<a href="#" class="pagenav to-start">&lsaquo; <__trans phrase="Prev" escape="js"></a>');
  } else {
    \$e.append('<span class="pagenav start disabled">&laquo; <__trans phrase="First" escape="js"></span>')
      .append('<span class="pagenav to-start disabled">&lsaquo; <__trans phrase="Prev" escape="js"></span>');
  }
  var end = (page == last) ? total : start+limit-1;
  var page_status = trans('[_1] - [_2] of [_3]', start, end, total);

  \$e.append('<span class="current-rows">'+page_status+'</span>')
  if (uselast) {
    \$e.append('<a href="#" class="pagenav to-end"><__trans phrase="Next" escape="js"> &rsaquo;</a>')
      .append('<a href="#" class="pagenav end"><__trans phrase="Last" escape="js"> &raquo;</a>');
  } else {
    \$e.append('<span class="pagenav to-end disabled"><__trans phrase="Next" escape="js"> &rsaquo;</span>')
      .append('<span class="pagenav end disabled"><__trans phrase="Last" escape="js"> &raquo;</span>');
  }
  jQuery('a.start').click(function() {
    saveChecked(page);
    renderList('filtered_list', cols, vals, jQuery('#row').val(), 1);
    return false;
  });
  jQuery('a.to-start').click(function() {
    saveChecked(page);
    renderList('filtered_list', cols, vals, jQuery('#row').val(), page-1);
    return false;
  });
  jQuery('a.to-end').click(function() {
    saveChecked(page);
    renderList('filtered_list', cols, vals, jQuery('#row').val(), page+1);
    return false;
  });
  jQuery('a.end').click(function() {
    saveChecked(page);
    renderList('filtered_list', cols, vals, jQuery('#row').val(), last);
    return false;
  });
}

HTML

    $node->innerHTML($html);
    $tmpl->insertBefore($node, $field);

#    my $plugin = MT->component("ListingFieldEditor");
#    my $node2 = $tmpl->createElement('setvarblock', { name => 'content_header', append => 1 });
#    my $html2 = 'Can use ' . rest_day() . 'days';
#    $node2->innerHTML($html2);
#    $tmpl->insertBefore($node2, $field);

    1;
}

sub add_css_cd {
    my ($cb, $app, $template) = @_;

    my $q = $app->param;

    my $blog_id = $app->param('blog_id') || 0;
    return if !$blog_id;

    my $mode = $q->param('__mode');
    return if !($mode eq 'list' || $mode eq 'filtered_list' || $mode eq 'save_from_list' || $mode eq 'save_from_list_cd');

    my $type = $q->param('_type');
    return if !($type eq 'content_data');

    my $asset_type = [qw( file audio image video)] if $type eq 'asset';

    my $q = $app->param;

    my $plugin = $app->component('PowerListingFieldEditor');
    my $text_width = $plugin->get_config_value('text_width', 'blog:'.$app->param('blog_id')) || 250;
    my $textarea_width = $plugin->get_config_value('textarea_width', 'blog:'.$app->param('blog_id')) || 400;
    my $text = $text_width - 15;
    my $textarea = $textarea_width - 20;

    my @text_selectors;
    my @textarea_selectors;
    my @button_selectors;
    my @button_selectors2;

if ( $type eq 'content_data' ) {
    my $content_type_id = $q->param('type');
    $content_type_id =~ s/^content_data_(.*)$/$1/;

    my $content_type = MT::ContentType->load({ blog_id => $app->blog->id, id => $content_type_id });

    for my $field_data ( @{ $content_type->fields } ) {
        my $type = $field_data->{type};

        next if !( $type eq 'single_line_text' ||
                   $type eq 'multi_line_text' ||
                   $type eq 'checkboxes' ||
                   $type eq 'select_box' ||
                   $type eq 'radio_button' ||
                   $type eq 'number' ||
                   $type eq 'date_and_time' ||
                   $type eq 'date_only' ||
                   $type eq 'time_only' ||
                   $type eq 'url' ||
                   $type eq 'image' );
        my $bn = $field_data->{id};
        if ( $type eq 'single_line_text' ||
             $type eq 'url' ||
             $type eq 'number' ||
             $type eq 'date_and_time' ||
             $type eq 'date_only' ||
             $type eq 'time_only' ) {
            push @text_selectors, ".col.head.blog" . $blog_id . "_" . $bn;
        } elsif ( $type eq 'multi_line_text' ) {
            push @textarea_selectors, ".col.head.blog" . $blog_id . "_" . $bn;
        } elsif ( $type eq 'checkboxes' || $type eq 'radio_button' || $type eq 'select_box' || $type eq 'image' ) {
            push @button_selectors, ".col.head.blog" . $blog_id . "_" . $bn;
            if ( $type eq 'radio' ) {
            push @button_selectors2, "td.col.blog" . $blog_id . "_" . $bn . " .buttons";
            }
        }
    }

} else {

    my @fields = CustomFields::Field->load({
        obj_type => ($asset_type) ? $asset_type : $type eq 'member' ? 'author' : $type,
        blog_id => $blog_id ? [ $blog_id, 0 ] : 0,
    });
    return if !@fields;

    my $plugin = $app->component('ListingFieldEditor');
    my $text_width = $plugin->get_config_value('text_width', 'blog:'.$app->param('blog_id')) || 250;
    my $textarea_width = $plugin->get_config_value('textarea_width', 'blog:'.$app->param('blog_id')) || 400;
    my $text = $text_width - 15;
    my $textarea = $textarea_width - 20;

    my @text_selectors;
    my @textarea_selectors;
    my @button_selectors;
    my @button_selectors2;

    foreach my $field (@fields) {
        next if !( $field->type eq 'text' ||
                   $field->type eq 'textarea' ||
                   $field->type eq 'checkbox' ||
                   $field->type eq 'select' ||
                   $field->type eq 'radio' ||
                   $field->type eq 'url' ||
                   $field->type eq 'image' );
        ( my $bn = $field->basename ) =~ s/-/_LFE_/;
        if ( $field->type eq 'text' || $field->type eq 'url' ) {
            push @text_selectors, ".col.head.blog" . $blog_id . "_" . $bn;
        } elsif ( $field->type eq 'textarea' ) {
            push @textarea_selectors, ".col.head.blog" . $blog_id . "_" . $bn;
        } elsif ( $field->type eq 'checkbox' || $field->type eq 'radio' || $field->type eq 'select' || $field->type eq 'image' ) {
            push @button_selectors, ".col.head.blog" . $blog_id . "_" . $bn;
            if ( $field->type eq 'radio' ) {
            push @button_selectors2, "td.col.blog" . $blog_id . "_" . $bn . " .buttons";
            }
        }
    }
}
    my $text_selector = join(",", @text_selectors) . "{width: ${text_width}px;}" if @text_selectors;
    my $textarea_selector = join(",", @textarea_selectors) . "{width: ${textarea_width}px;}" if @textarea_selectors;

    my $button_selector = join(",", @button_selectors) . "{width: 140px;}" if @button_selectors;
    my $button_selector2 = join(",", @button_selectors2) . "{display: block; margin-top: 3px;}" if @button_selectors2;

    my $old;
    my $new;

    $old = <<HTML;
  <mt:var name="html_head">
HTML
    $old = quotemeta($old);
    $new = <<HTML;
<style type="text/css">
$text_selector
$textarea_selector
$button_selector
$button_selector2

a.cancel-edit, a.submit-note {
    padding: 3px 8px;
    text-decoration: none;
    color: #000;
}

.lfetext, .lfetextarea, .lfeurl {
    -moz-border-bottom-colors: none;
    -moz-border-image: none;
    -moz-border-left-colors: none;
    -moz-border-right-colors: none;
    -moz-border-top-colors: none;
    -moz-box-sizing: border-box;
    background-color: #FBFBFB;
    border-color: #ADADAD #DCDDDD #DCDDDD #ADADAD;
    border-radius: 3px 3px 3px 3px;
    border-right: 1px solid #DCDDDD;
    border-style: solid;
    border-width: 1px;
    box-shadow: 0 1px 0 #F3F3F3;
    font-size: 13px;
    outline: 0 none;
    padding: 0.2em 0.25em;
    vertical-align: middle;
    margin-bottom: 3px;
}
.lfetext, .lfeurl {
    width: ${text}px;
}
.lfetextarea {
    vertical-align: bottom;
    height: 100px;
    width: ${textarea}px;
}
.lfecheckbox {
    margin-bottom: 3px;
}
.lfeselect {
    margin-bottom: 3px;
}
.lferadio .submit-note, .lferadio .cancel-note {
    margin-bottom: 3px;
}
.image-link {
    margin: 3px 0;
}
</style>
  <mt:var name="html_head">
HTML

    $$template =~ s/$old/$new/;

    $old = <<HTML;
  <script type="text/javascript" src="<\$mt:var name="static_uri"\$>jqueryui/jquery-ui.js?v=<\$mt:var name="mt_version_id" escape="url"\$>"></script>
HTML
    $old = quotemeta($old);
    $new = <<HTML;
  <script type="text/javascript" src="<\$mt:var name="static_uri"\$>plugins/PowerListingFieldEditor/js/jquery-ui.js?v=<\$mt:var name="mt_version_id" escape="url"\$>"></script>
HTML
    $$template =~ s/$old/$new/;

    $old = <<HTML;
  <script type="text/javascript" src="<\$mt:var name="static_uri"\$>jqueryui/jquery-ui.min.js?v=<\$mt:var name="mt_version_id" escape="url"\$>"></script>
HTML
    $old = quotemeta($old);
    $new = <<HTML;
  <script type="text/javascript" src="<\$mt:var name="static_uri"\$>plugins/PowerListingFieldEditor/js/jquery-ui.js?v=<\$mt:var name="mt_version_id" escape="url"\$>"></script>
HTML
    $$template =~ s/$old/$new/;
}

sub delete_js2 {
    my ($cb, $app, $template) = @_;

    my $q = $app->param;
    my $mode = $q->param('__mode');
    my $type = $q->param('_type');
    my $blog_id = $q->param('blog_id') || 0;

    return if !$blog_id;
    return if $mode ne 'list';
    return if $type ne 'entry' && $type ne 'page' && $type ne 'content_data';

    my $old;
    my $new;

if ( $type eq 'entry' || $type eq 'page' ) {
    $old = <<HTML;
\$checked.parents('tr').addClass('selected');
HTML
    $old = quotemeta($old);
    $new = <<HTML;
//\$checked.parents('tr').addClass('selected');
HTML
#    $$template =~ s/$old/$new/;
}

    $old = <<HTML;
<script type="text/javascript" src="<mt:var name="static_uri">js/listing/list_data.js?v=<mt:var name="mt_version_id" escape="url">"></script>
HTML
    $old = quotemeta($old);
    $new = <<HTML;
<script type="text/javascript" src="<mt:var name="static_uri">plugins/PowerListingFieldEditor/js/list_data.js?v=<mt:var name="mt_version_id" escape="url">"></script>
HTML
    $$template =~ s/$old/$new/;

    $old = <<HTML;
  <script type="text/javascript" src="<mt:var name="static_uri">js/listing/listing.js?v=<mt:var name="mt_version_id" escape="url">"></script>
HTML
    $old = quotemeta($old);
    $new = <<HTML;
  <script type="text/javascript" src="<mt:var name="static_uri">js/listing/listing.js?v=<mt:var name="mt_version_id" escape="url">"></script>
  <script type="text/javascript" src="<mt:var name="static_uri">plugins/PowerListingFieldEditor/js/add.js?v=<mt:var name="mt_version_id" escape="url">"></script>
  <script type="text/javascript" src="<mt:var name="static_uri">js/editor.js"></script>
HTML
    $$template =~ s/$old/$new/;

    $old = <<HTML;
<mt:setvarblock name="system_msg" append="1">
<div id="msg-block"></div>
</mt:setvarblock>
HTML
    $old = quotemeta($old);
    $new = <<HTML;
<mt:setvarblock name="system_msg" append="1">
<div id="msg-block"></div>
<div style="margin-top:20px">
<mt:var name="rest_day">
</div>
</mt:setvarblock>
HTML
    $$template =~ s/$old/$new/;

}

sub bulk_sort_cd {
    my $prop = shift;
    my ($objs) = @_;
    my $basename = $prop->content_field_id;
    my %ids = map { $_->id => 1 } @$objs;
    my @content_data_list = MT::ContentData->load( { id => [ keys %ids ] } );
    my %data = map { $_->id => $_->data->{ $prop->content_field_id } } @content_data_list;
    return sort {
        $data{ $a->id } cmp $data{ $b->id }
    } @$objs;
}

sub _default_sort {
    my $prop = shift;
    my ( $terms, $args ) = @_;

    my $cf_idx_join = MT::ContentFieldIndex->join_on(
        undef, undef,
        {   type      => 'left',
            condition => {
                content_data_id  => \'= cd_id',
                content_field_id => $prop->content_field_id,
            },
            sort      => 'value_' . $prop->data_type,
            direction => delete $args->{direction},
            unique    => 1,
        },
    );

    $args->{joins} ||= [];
    push @{ $args->{joins} }, $cf_idx_join;

    return;
}

#####################################################################################################################################################

sub bulk_sort {
    my $prop = shift;
    my ($objs) = @_;
    my $basename = $prop->basename;
    my %entry_id = map { $_->id => 1 } @$objs;
    my @entries = MT->model('entry')->load( { id => [ keys %entry_id ] } );
    my %data = map { $_->id => $_->meta('field.' . $basename) } @entries;
    return sort {
        $data{ $a->id } cmp $data{ $b->id }
    } @$objs;
}

sub out_html_default_status {
    my ( $prop, $obj ) = @_;

    my $meta = $obj->status;
    my $status_label = MT->translate( MT::Entry::status_text( $obj->status ) );
    $meta = '---' if !$meta;
    my $id = $obj->id;

    my $options = $prop->single_select_options;
    my $buf;
    $buf .= qq{<option value=""></option>};
    for my $option (@{$options}) {
        my $value = $option->{value};
        my $label = $option->{label};
        if ( $value eq $meta ) {
            $buf .= qq{<option value="$value" selected="selected">$label</option>};
        } else {
            $buf .= qq{<option value="$value">$label</option>};
        }
    }
    my $element;
    $element = qq{<select name="note-status-$id" id="note-status-$id" class="lfeselect">} . $buf . qq{</select>};

    my $save_changes_label = MT->translate('Save Changes');
    my $save_label = MT->translate('Save');
    my $cancel_label = MT->translate('Cancel');

    my $out;
    $out = qq{<span id="note-status-link-$id" class="view-link"><a href="#$id" class="start-edit" title="}.encode_html($meta).qq{">}.$status_label.qq{</a></span><span id="note-status-field-$id" style="display: none">} . $element . qq{<span class="buttons"><a href="#$id" title="$save_changes_label" type="button" class="button submit-note">$save_label</a><a href="#$id" type="button" class="button cancel-edit">$cancel_label</a></span></span>};
    return $out;
}

sub out_html_default_category {
    my ( $prop, $obj ) = @_;

    my $id = $obj->id;
    my ( $args, $terms, $join_terms, $join_args );
    $join_terms->{ entry_id } = $id;
    $join_terms->{ is_primary } = 1;
    $args->{ join } = MT::Placement->join_on( 'category_id', $join_terms );
    $terms->{ blog_id } = MT->app->blog->id;

    my @categories = MT->model( $prop->category_class )->load( $terms, $args );
    my $meta;
    my $label;
    for my $cat (@categories) {
        $label = $cat->label;
        $meta = $cat->id;
    }
    $label = '---' if !$meta;

    my $options = $prop->single_select_options;
    my $buf;
    $buf .= qq{<option value=""></option>} if !$meta;
    for my $option (@{$options}) {
        my $value = $option->{value};
        my $label = $option->{label};
        if ( $value eq $meta ) {
            $buf .= qq{<option value="$value" selected="selected">$label</option>};
        } else {
            $buf .= qq{<option value="$value">$label</option>};
        }
    }
    my $element;
    $element = qq{<select name="note-category-$id" id="note-category-$id" class="lfeselect">} . $buf . qq{</select>};

    my $save_changes_label = MT->translate('Save Changes');
    my $save_label = MT->translate('Save');
    my $cancel_label = MT->translate('Cancel');

    my $out;
    $out = qq{<span id="note-category-link-$id" class="view-link"><a href="#$id" class="start-edit" title="}.encode_html($meta).qq{">}.$label.qq{</a></span><span id="note-category-field-$id" style="display: none">} . $element . qq{<span class="buttons"><a href="#$id" title="$save_changes_label" type="button" class="button submit-note">$save_label</a><a href="#$id" type="button" class="button cancel-edit">$cancel_label</a></span></span>};
    return $out;
}

sub out_html_default_asset {
    my ( $prop, $obj ) = @_;

    my $select = MT->translate('Select');
    my $delete = MT->translate('Delete');

    my $id = $obj->id;
    my @object_assets = MT::ObjectAsset->load({ object_ds => 'entry', object_id => $id } );

    my $save_changes_label = MT->translate('Save Changes');
    my $save_label = MT->translate('Save');
    my $cancel_label = MT->translate('Cancel');
    my $element;

    my $meta = '';

    my $asset_id = '';
    my @asset_ids;
    for my $object_asset (@object_assets) {
        my @assets = MT->model('asset')->load( { id => $object_asset->asset_id } );
        for (@assets) {
            my ( $url, $w, $h ) = $_->thumbnail_url( Height => 45, Width => 45, Square => 1 );
            $meta .= '<img id="image-' . 'include_asset_ids' . '-' . $object_asset->asset_id . '-child" src="'.$url.'" />';
            $asset_id = $_->id;
            push @asset_ids, $_->id;
        }
    }
    $meta = '---' if !@object_assets;

    my $uri = MT->instance->app_uri;
    my $blog_id = MT->instance->blog->id;
    $element = qq{<div id="image-include_asset_ids-$id">$meta</div><div class="image-link"><a class="mt-open-dialog" href="${uri}?__mode=list_asset_for_list&_type=asset&blog_id=$blog_id&obj_id=$id&basename=include_asset_ids&dialog_view=1&filter=class&filter_val=image&require_type=image&edit_field=customfield_cfimage&asset_select=1'">} . $select . qq{</a> <a href="#$id" class="clear-image" id="delete-include_asset_ids-$id">} . $delete . qq{</a></div>};

    my $out;
    $out = qq{<span id="note-include_asset_ids-link-$id" class="view-link"><a href="#$id" class="start-edit" title="">}. $meta .qq{</a></span><span id="note-include_asset_ids-field-$id" style="display: none">} . $element . qq{<input class="include_asset_ids-$id" id="include_asset_ids-include_asset_ids-$id" type="hidden" value="$asset_id" name="include_asset_ids" /><input id="note-include_asset_ids-$id" class="asset_ids-$id" type="hidden" value="} . join(',',@asset_ids) . qq{" name="include_asset_ids" /><span class="buttons"><a href="#$id" title="$save_changes_label" type="button" class="button submit-note">$save_label</a><a href="#$id" type="button" class="button cancel-edit">$cancel_label</a></span></span>};
    return $out;
}

sub out_html_default_title {
    my ( $prop, $obj ) = @_;

    my $class       = $obj->class;
    my $class_label = $obj->class_label;
#    my $title       = $prop->super(@_);

    my $value = $obj->title;
    $value = '---' if !$value;
    my $id = $obj->id;
    my $save_changes_label = MT->translate('Save Changes');
    my $save_label = MT->translate('Save');
    my $cancel_label = MT->translate('Cancel');
    my $element = qq{<input type="text" name="note-title-$id" id="note-title-$id" class="lfetext" value="} . encode_html($value) . qq{" />};
    my $title;
    $title = qq{<span id="note-title-link-$id" class="view-link"><a href="#$id" class="start-edit" title="}.encode_html($value).qq{">}.encode_html($value).qq{</a></span><span id="note-title-field-$id" style="display: none">} . $element . qq{<span class="buttons"><a href="#$id" title="$save_changes_label" type="button" class="button submit-note">$save_label</a><a href="#$id" type="button" class="button cancel-edit">$cancel_label</a></span></span>};


#    my $id        = $obj->id;
    my $permalink = MT::Util::encode_html( $obj->permalink );
    my $edit_url  = MT->app->uri(
        mode => 'view',
        args => {
            _type   => $class,
            id      => $obj->id,
            blog_id => $obj->blog_id,
        }
    );
    my $status = $obj->status;
    my $status_class
        = $status == MT::Entry::HOLD()    ? 'Draft'
        : $status == MT::Entry::RELEASE() ? 'Published'
        : $status == MT::Entry::REVIEW()  ? 'Review'
        : $status == MT::Entry::FUTURE()  ? 'Future'
        : $status == MT::Entry::JUNK()    ? 'Junk'
        :                                   '';
    my $lc_status_class = lc $status_class;
    require MT::Entry;
    my $status_file
        = $status == MT::Entry::HOLD()    ? 'draft.gif'
        : $status == MT::Entry::RELEASE() ? 'success.gif'
        : $status == MT::Entry::REVIEW()  ? 'warning.gif'
        : $status == MT::Entry::FUTURE()  ? 'future.gif'
        : $status == MT::Entry::JUNK()    ? 'warning.gif'
        :                                   '';
    my $status_img
        = MT->static_path . 'images/status_icons/' . $status_file;
    my $view_img
        = MT->static_path . 'images/status_icons/view.gif';
    my $view_link = $obj->status == MT::Entry::RELEASE()
        ? qq{
        <span class="view-link2">
          <a href="$permalink" target="_blank">
            <img alt="View $class_label" src="$view_img" />
          </a>
        </span>
    }
        : '';

    my $out = qq{
        <span class="icon status2 $lc_status_class">
          <a href="$edit_url"><img alt="$status_class" src="$status_img" /></a>
        </span>
        <span class="title">
          $title
        </span>
        $view_link
    };
    return $out;
}

sub out_html_default {
    my ( $prop, $obj ) = @_;

    my $col = $prop->column eq 'title2' ? 'title' : $prop->column;
#    my $col = $prop->column;

my $value;
my $form_value;
if ($col eq 'tags') {
    my $tag_delim = chr( MT->app->user->entry_prefs->{tag_delim} );
    require MT::Tag;
    $form_value = $value = MT::Tag->join( $tag_delim, $obj->tags );
} else {
    $form_value = $value = $obj->$col;
}
#    my $value = $obj->$col;
#    my $form_value = $obj->$col;
    if ($prop->type ne 'checkbox') {
        $form_value = ''  if !$value;
        $value = '---' if !$value;
    } else {
        $value = '0' if !$value;
    }
    my $id = $obj->id;

    my $save_changes_label = MT->translate('Save Changes');
    my $save_label = MT->translate('Save');
    my $cancel_label = MT->translate('Cancel');
    my $element;
    my $label;
    if ($prop->type eq 'text') {
        $label = $value;
        $element = qq{<input type="text" name="note-$col-$id" id="note-$col-$id" class="lfetext" value="} . encode_html($form_value) . qq{" />};
    } elsif ($prop->type eq 'date') {
        my $plugin = MT->component('PowerListingFieldEditor');
        my $y = $plugin->translate('year');
        my $m = $plugin->translate('month');
        my $d = $plugin->translate('day');
        my $h = $plugin->translate('hour');
        my $min = $plugin->translate('minuite');
        my $s = $plugin->translate('second');
        ($label = $value) =~ s/(\d{4})(\d{2})(\d{2})(\d{2})(\d{2})(\d{2})/$1$y$2$m$3$d/;
        $value =~ s/(\d{4})(\d{2})(\d{2})(\d{2})(\d{2})(\d{2})/$1-$2-$3 $4:$5:$6/;
        $element = qq{<input type="text" name="note-$col-$id" id="note-$col-$id" class="lfedate" value="} . encode_html($value) . qq{" />};
    } elsif ($prop->type eq 'textarea') {
        $label = $value;
        $element = qq{<textarea name="note-$col-$id" id="note-$col-$id" class="lfetextarea">} . encode_html($form_value) . qq{</textarea>};
    }
    my $out;
    $out = qq{<span id="note-$col-link-$id" class="view-link"><a href="#$id" class="start-edit" title="}.encode_html($value).qq{">}.encode_html($label).qq{</a></span><span id="note-$col-field-$id" style="display: none">} . $element . qq{<span class="buttons"><a href="#$id" title="$save_changes_label" type="button" class="button submit-note">$save_label</a><a href="#$id" type="button" class="button cancel-edit">$cancel_label</a></span></span>};
    return $out;
}

sub bulk_sort_default {
    my $prop = shift;
    my ($objs) = @_;
    my $col = $prop->column eq 'title2' ? 'title' : $prop->column;
#    my $col = $prop->column;
    my %entry_id = map { $_->id => 1 } @$objs;
    my @entries = MT->model('entry')->load( { id => [ keys %entry_id ] } );
    my %data = map { $_->id => $_->$col } @entries;
    return sort {
        $data{ $a->id } cmp $data{ $b->id }
    } @$objs;
}

sub out_html {
    my ( $prop, $obj ) = @_;

    my $select = MT->translate('Select');
    my $delete = MT->translate('Delete');

    my $basename = $prop->basename;
    my $meta = $obj->meta('field.' . $basename);
    if ($prop->type ne 'checkbox') {
        $meta = '---' if !$meta;
    } else {
        $meta = '0' if !$meta;
    }
    my $id = $obj->id;

    my $asset_id;
    my $orig_meta;

    my $save_changes_label = MT->translate('Save Changes');
    my $save_label = MT->translate('Save');
    my $cancel_label = MT->translate('Cancel');
    my $element;
    if ($prop->type eq 'text') {
        $element = qq{<input type="text" name="note-$basename-$id" id="note-$basename-$id" class="lfetext" value="} . encode_html($meta) . qq{" />};
    } elsif ($prop->type eq 'textarea') {
        $element = qq{<textarea name="note-$basename-$id" id="note-$basename-$id" class="lfetextarea">} . encode_html($meta) . qq{</textarea>};
    } elsif ($prop->type eq 'url') {
        $element = qq{<input type="text" name="note-$basename-$id" id="note-$basename-$id" class="lfeurl" value="} . encode_html($meta) . qq{" />};
    } elsif ($prop->type eq 'checkbox') {
        my $flag = $meta eq '0' ? '' : 'checked="checked"';
        $element = qq{<div><input type="checkbox" name="note-$basename-$id" id="note-$basename-$id" class="lfecheckbox" } . $flag . qq{ /></div>};
    } elsif ($prop->type eq 'select') {
        my @options = split(',', $prop->option);
        my $buf;
        $buf .= qq{<option value=""></option>};
        for my $option (@options) {
            if ( $option eq $meta ) {
                $buf .= qq{<option value="$option" selected="selected">$option</option>};
            } else {
                $buf .= qq{<option value="$option">$option</option>};
            }
        }
        $element = qq{<select name="note-$basename-$id" id="note-$basename-$id" class="lfeselect">} . $buf . qq{</select>};
    } elsif ($prop->type eq 'radio') {
        my @options = split(',', $prop->option);
        for my $option (@options) {
            if ( $option eq $meta ) {
                $element .= qq{<input type="radio" name="note-$basename-$id" value="$option" checked="checked" /> } . encode_html($option) . '<br />';
            } else {
                $element .= qq{<input type="radio" name="note-$basename-$id" value="$option" /> } . encode_html($option) . '<br />';
            }
        }
        $element = qq{<div id="note-$basename-$id">} . $element . qq{</div>};
    } elsif ($prop->type eq 'image') {
        $orig_meta = $meta;
        $meta =~ s/.*asset-id="(\d+)".*/$1/;
        my @assets = MT->model('asset')->load( { id => $meta } );
        for (@assets) {
            my ( $url, $w, $h ) = $_->thumbnail_url( Height => 45, Width => 45, Square => 1 );
            $meta = '<img id="image-' . $basename . '-' . $id . '-child" src="'.$url.'" />';
$asset_id = $_->id;
        }
        my $uri = MT->instance->app_uri;
        my $blog_id = MT->instance->blog->id;
        $element = qq{<div id="image-$basename-$id">$meta</div><div class="image-link"><a class="mt-open-dialog" href="${uri}?__mode=list_asset_for_list&_type=asset&blog_id=$blog_id&obj_id=$id&basename=$basename&dialog_view=1&filter=class&filter_val=image&require_type=image&edit_field=customfield_cfimage&asset_select=1'">} . $select . qq{</a> <a href="#$id" class="clear-image" id="delete-$basename-$id">} . $delete . qq{</a></div>};
    }
    my $out;
    if ($prop->type ne 'image') {
        $out = qq{<span id="note-$basename-link-$id" class="lfe-view-link"><a href="#$id" class="start-edit" title="}.encode_html($meta).qq{">}.encode_html($meta).qq{</a></span><span id="note-$basename-field-$id" style="display: none">} . $element . qq{<span class="buttons"><a href="#$id" title="$save_changes_label" type="button" class="button submit-note">$save_label</a><a href="#$id" type="button" class="button cancel-edit">$cancel_label</a></span></span>};
    } else {
        $out = qq{<span id="note-$basename-link-$id" class="lfe-view-link"><a href="#$id" class="start-edit" title="">}. $meta .qq{</a></span><span id="note-$basename-field-$id" style="display: none">} . $element . qq{<input class="include_asset_ids-$id" id="include_asset_ids-$basename-$id" type="hidden" value="$asset_id" name="include_asset_ids" /><input id="note-$basename-$id" class="customfield_$basename-$id" type="hidden" value="} . encode_html($orig_meta) . qq{" name="customfield_$basename" /><span class="buttons"><a href="#$id" title="$save_changes_label" type="button" class="button submit-note">$save_label</a><a href="#$id" type="button" class="button cancel-edit">$cancel_label</a></span></span>};
    }
    return $out;
}

sub add_script {
    my ($cb, $app, $param, $tmpl) = @_;
    my $type = $app->param('_type');
    my $blog_id = $app->blog->id;
    return if is_limit();

    my $asset_type = [qw( file audio image video)] if $type eq 'asset';

    my @fields = CustomFields::Field->load({
        obj_type => ($asset_type) ? $asset_type : $type eq 'member' ? 'author' : $type,
        blog_id => $app->blog ? [ $blog_id, 0 ] : 0,
    });
#    return if !@fields;

#    $type = 'member' if $type eq 'author';

    my $old_buf;
    my $link_buf;
    my $field_buf;
    my $note_buf;
    my $old_note_buf;
    my $empty_note_buf;
    my $empty_old_note_buf;
    my $recover_buf;
    my $clear_note_buf;
    my $new_note_buf;
    my @cancel_buf;
    my $submit_buf;
    my $render_buf;

    my $all_note_buf;
    my $all_old_note_buf;
    my $all_empty_note_buf;
    my $all_empty_old_note_buf;
    my $all_recover_buf;
    my $all_clear_note_buf;
    my $all_new_note_buf;
    my @all_cancel_buf;
    my $all_submit_buf;
    my $all_render_buf;

    my @basenames;
    my $counter = 0;

    my $image_buf;
    my $old_image_buf;
    my $recover_image_buf;
    my $clear_image_buf;
    my $asset_ids;

    my $all_image_buf;
    my $all_old_image_buf;
    my $all_recover_image_buf;
    my $all_clear_image_buf;
    my $all_asset_ids;

    my $all_note_buf_new;
    my $all_old_note_buf_new;
    my $all_check_note_buf_new;

    foreach my $field (@fields) {
        next if !( $field->type eq 'text'     ||
                   $field->type eq 'textarea' ||
                   $field->type eq 'checkbox' ||
                   $field->type eq 'select'   ||
                   $field->type eq 'radio'    ||
                   $field->type eq 'url'      ||
                   $field->type eq 'image' );
            $all_note_buf_new .= "all_note_" . $field->basename . " = new Array();\n";
            $all_old_note_buf_new .= "all_old_note_" . $field->basename . " = new Array();\n";
            $all_check_note_buf_new .= "all_check_note_" . $field->basename . " = new Array();\n";
    }

    foreach my $col (qw( title text excerpt text_more keywords include_asset_ids status authored_on modified_on category tags )) {
        $old_buf .=  "var old_note_" . $col . " = '';\nvar check_note_" . $col . " = '';\n";
        $link_buf .=  "note_links.push( jQuery('#note-" . $col . "-link-' + id) );\n";
        $field_buf .= "note_fields.push( jQuery('#note-" . $col . "-field-' + id) );\n";

        # note
        $note_buf .= "var note_" . $col . " = jQuery('#note-" . $col . "-' + id);\n";

        # old
        if ($col eq 'allow_comments' || $col eq 'allow_pings') {
            $old_note_buf .= "old_note_" . $col . " =  jQuery('#note-" . $col . "-' + id).attr('checked');";
        } else {
            $old_note_buf .= "old_note_" . $col . " = note_" . $col . ".val();\ncheck_note_" . $col . " = note_" . $col . ".val();\n";
        }

        # empty
        if (!($col eq 'allow_comments' || $col eq 'allow_pings')) {
            $empty_note_buf .= "if (note_" . $col . ".val() == '---'){\n  note_" . $col . ".val('');\n  check_note_" . $col . " = '';\n}\n";
        } else {
            $empty_note_buf .= "if (jQuery('#note-" . $col . "-' + id).attr('checked') == true) { note_" . $col . "= 1;\ncheck_note_" . $col . "= 1; } else { note_" . $col . "= 0;\ncheck_note_" . $col . "= 0; }";
        }

        # recover
        if (!($col eq 'allow_comments' || $col eq 'allow_pings')) {
            $recover_buf .= "note_" . $col . ".val( old_note_" . $col . " );\n";
        } else {
            $recover_buf .= "note_" . $col . ".attr('checked', old_note_" . $col . ");\n";
        }
        $clear_note_buf .=  "old_note_" . $col . " = '';\n";

        # new
        if ($col eq 'allow_comments' || $col eq 'allow_pings') {
            $new_note_buf .= "var new_note_" . $col . ";\nif (jQuery('#note-" . $col . "-' + id).attr('checked') == true) { new_note_" . $col . "= 1; } else { new_note_" . $col . "= 0; }";
        } else {
            $new_note_buf .= "var new_note_" . $col . " = note_" . $col . ".val();\n";
        }

        push @cancel_buf, "check_note_" . $col . " == new_note_" . $col;

        # submit
        if ($col eq 'allow_comments' || $col eq 'allow_pings') {
            $submit_buf .= "var note_" . $col . ";\nif (jQuery('#note-" . $col . "-' + id).attr('checked') == true) { note_" . $col . "= 1; } else { note_" . $col . "= 0; }";
            $all_submit_buf .= "var note_" . $col . ";\nif (jQuery('#note-" . $col . "-' + id).attr('checked') == true) { note_" . $col . "= 1; } else { note_" . $col . "= 0; }";
        } else {
            $submit_buf .= "var note_" . $col . " = jQuery('#note-" . $col . "-' + id).val();\n";
            $all_submit_buf .= "var note_" . $col . " = jQuery('#note-" . $col . "-' + id).val();\n";
        }

        # render
        if ($col eq 'allow_comments' || $col eq 'allow_pings') {
            $render_buf .= $col . "_cb_beacon: 1,\n";
#            $all_render_buf .= "param.append('" . $col . "_cb_beacon', 1);\n";
#            $all_render_buf .= $col . "_cb_beacon: 1,\n";
            $all_render_buf .= "param['" . $col . "_cb_beacon'] = 1;\n";
        }
        $render_buf .= $col . ": note_" . $col . ",\n";
#        $all_render_buf .= "param.append('" . $col . "', note_" . $col . ");\n";
#        $all_render_buf .= $col . ": note_" . $col . ",\n";
        $all_render_buf .= "param['" . $col . "'] =  note_" . $col . ";\n";

        $counter++;
    }

    $old_image_buf .= "var old_image_" . 'include_asset_ids' . " = '';\n";
    $image_buf .= "old_image_" . 'include_asset_ids' . " = jQuery('#image-" . 'include_asset_ids' . "-' + id).html();\n";
    $recover_image_buf .= "if(old_note_" . 'include_asset_ids' . " == '---') {\nold_note_" . 'include_asset_ids' . " = '';\n}\njQuery('#image-" . 'include_asset_ids' . "-' + id).html( old_image_" . 'include_asset_ids' . " );\nif (old_image_" . 'include_asset_ids' . " == '---'){\n  jQuery('#delete-" . 'include_asset_ids' . "-' + id).css('display', 'none');\n} else {\n  jQuery('#delete-" . 'include_asset_ids' . "-' + id).css('display', 'inline');\n}\n";
    $clear_image_buf .=  "old_image_" . 'include_asset_ids' . " = '';\n";

    foreach my $field (@fields) {
        next if !( $field->type eq 'text'     ||
                   $field->type eq 'textarea' ||
                   $field->type eq 'checkbox' ||
                   $field->type eq 'select'   ||
                   $field->type eq 'radio'    ||
                   $field->type eq 'url'      ||
                   $field->type eq 'image' );
        $old_buf .=  "var old_note_" . $field->basename . " = '';\nvar check_note_" . $field->basename . " = '';\n";
        $link_buf .=  "note_links.push( jQuery('#note-" . $field->basename . "-link-' + id) );\n";
        $field_buf .= "note_fields.push( jQuery('#note-" . $field->basename . "-field-' + id) );\n";

        # note
        if ($field->type eq 'radio') {
            $note_buf .= "var note_" . $field->basename . " = jQuery('[name=\"note-" . $field->basename . "-' + id + '\"]:checked');\n";
            $all_note_buf .= "all_note_" . $field->basename . "[id] = jQuery('[name=\"note-" . $field->basename . "-' + id + '\"]:checked');\n";
        } else {
            $note_buf .= "var note_" . $field->basename . " = jQuery('#note-" . $field->basename . "-' + id);\n";
            $all_note_buf .= "all_note_" . $field->basename . "[id] = jQuery('#note-" . $field->basename . "-' + id);\n";
        }

        # old
        if ($field->type eq 'checkbox') {
            $old_note_buf .= "old_note_" . $field->basename . " =  jQuery('#note-" . $field->basename . "-' + id).attr('checked');";
            $all_old_note_buf .= "all_old_note_" . $field->basename . "[id] =  jQuery('#note-" . $field->basename . "-' + id).attr('checked');";
        } else {
            $old_note_buf .= "old_note_" . $field->basename . " = note_" . $field->basename . ".val();\ncheck_note_" . $field->basename . " = note_" . $field->basename . ".val();\n";
            $all_old_note_buf .= "all_old_note_" . $field->basename . "[id] = all_note_" . $field->basename . "[id].val();\nall_check_note_" . $field->basename . "[id] = all_note_" . $field->basename . "[id].val();\n";
        }

        # empty
        if ($field->type ne 'checkbox') {
            $empty_note_buf .= "if (note_" . $field->basename . ".val() == '---'){\n  note_" . $field->basename . ".val('');\n  check_note_" . $field->basename . " = '';\n}\n";
            $all_empty_note_buf .= "if (all_note_" . $field->basename . "[id].val() == '---'){\n  all_note_" . $field->basename . "[id].val('');\n  all_check_note_" . $field->basename . "[id] = '';\n}\n";
        } else {
            $empty_note_buf .= "if (jQuery('#note-" . $field->basename . "-' + id).attr('checked') == 'checked' || jQuery('#note-" . $field->basename . "-' + id).attr('checked') == true) { note_" . $field->basename . "= 1;\ncheck_note_" . $field->basename . "= 1; } else { note_" . $field->basename . "= 0;\ncheck_note_" . $field->basename . "= 0; }";
            $all_empty_note_buf .= "if (jQuery('#note-" . $field->basename . "-' + id).attr('checked') == 'checked' || jQuery('#note-" . $field->basename . "-' + id).attr('checked') == true) { all_note_" . $field->basename . "[id] = 1;\nall_check_note_" . $field->basename . "[id] = 1; } else { all_note_" . $field->basename . "[id] = 0;\nall_check_note_" . $field->basename . "[id] = 0; }";
        }

        # recover
        if ($field->type ne 'checkbox') {
            $recover_buf .= "note_" . $field->basename . ".val( old_note_" . $field->basename . " );\n//alert(note_" . $field->basename . ".val());";
            $all_recover_buf .= "all_note_" . $field->basename . "[id].val( all_old_note_" . $field->basename . "[id] );\n//alert(all_note_" . $field->basename . "[id].val());";
        } else {
            $recover_buf .= "note_" . $field->basename . ".attr('checked', old_note_" . $field->basename . ");\n";
            $all_recover_buf .= "all_note_" . $field->basename . "[id].attr('checked', all_old_note_" . $field->basename . "[id] );\n";
        }
        $clear_note_buf .=  "old_note_" . $field->basename . " = '';\n";
        $all_clear_note_buf .=  "all_old_note_" . $field->basename . "[id] = '';\n";

        # new
        if ($field->type eq 'checkbox') {
#            $new_note_buf .= "var new_note_" . $field->basename . ";\nif (jQuery('#note-" . $field->basename . "-' + id).attr('checked') == true) { new_note_" . $field->basename . "= 1; } else { new_note_" . $field->basename . "= 0; }";
            $new_note_buf .= "var new_note_" . $field->basename . ";\nif (jQuery('#note-" . $field->basename . "-' + id).attr('checked') == 'checked' || jQuery('#note-" . $field->basename . "-' + id).attr('checked') == true) { new_note_" . $field->basename . "= 1; } else { new_note_" . $field->basename . "= 0; }";
#            $all_new_note_buf .= "all_new_note_" . $field->basename . "[id];\nif (jQuery('#note-" . $field->basename . "-' + id).attr('checked') == true) { all_new_note_" . $field->basename . "[id] = 1; } else { all_new_note_" . $field->basename . "[id] = 0; }";
            $all_new_note_buf .= "all_new_note_" . $field->basename . "[id];\nif (jQuery('#note-" . $field->basename . "-' + id).attr('checked') == 'checked' || jQuery('#note-" . $field->basename . "-' + id).attr('checked') == true) { all_new_note_" . $field->basename . "[id] = 1; } else { all_new_note_" . $field->basename . "[id] = 0; }";
        } else {
            $new_note_buf .= "var new_note_" . $field->basename . " = note_" . $field->basename . ".val();\n";
            $all_new_note_buf .= "all_new_note_" . $field->basename . "[id] = all_note_" . $field->basename . "[id].val();\n";
        }

        push @cancel_buf, "check_note_" . $field->basename . " == new_note_" . $field->basename;
        push @all_cancel_buf, "all_check_note_" . $field->basename . "[id] == all_new_note_" . $field->basename . "[id]";

        # submit
        if ($field->type eq 'radio') {
            $submit_buf .= "var note_" . $field->basename . " = jQuery('[name=\"note-" . $field->basename . "-' + id + '\"]:checked').val();\n";
            $all_submit_buf .= "var note_" . $field->basename . " = jQuery('[name=\"note-" . $field->basename . "-' + id + '\"]:checked').val();";
        } elsif ($field->type eq 'checkbox') {
            $submit_buf .= "var note_" . $field->basename . ";\nif (jQuery('#note-" . $field->basename . "-' + id)[0]) { if ( jQuery('#note-" . $field->basename . "-' + id).attr('checked') == 'checked' || jQuery('#note-" . $field->basename . "-' + id).attr('checked') == true) {note_" . $field->basename . "= 1; } else { note_" . $field->basename . "= 0; } } else { note_" . $field->basename . "= undefined; }";
            $all_submit_buf .= "var note_" . $field->basename . ";\nif (jQuery('#note-" . $field->basename . "-' + id)[0]) { if ( jQuery('#note-" . $field->basename . "-' + id).attr('checked') == 'checked' || jQuery('#note-" . $field->basename . "-' + id).attr('checked') == true) {note_" . $field->basename . "= 1; } else { note_" . $field->basename . "= 0; } } else { note_" . $field->basename . "= undefined; }";
        } else {
            $submit_buf .= "var note_" . $field->basename . " = jQuery('#note-" . $field->basename . "-' + id).val();\n";
            $all_submit_buf .= "var note_" . $field->basename . " = jQuery('#note-" . $field->basename . "-' + id).val();\n";
        }

        # render
        if ($field->type eq 'checkbox') {
#            $render_buf .= "customfield_" . $field->basename . "_cb_beacon: 1,\n";
#            $all_render_buf .= "customfield_" . $field->basename . "_cb_beacon: 1,";
#            $all_render_buf .= "param['customfield_" . $field->basename . "_cb_beacon'] =  1;";
        }
        $render_buf .= "customfield_" . $field->basename . ": note_" . $field->basename . ",\n";
#        $all_render_buf .= "param.append('customfield_" . $field->basename . "', note_" . $field->basename . ");\n";
#        $all_render_buf .= "customfield_" . $field->basename . ": note_" . $field->basename . ",\n";
        $all_render_buf .= "param['customfield_" . $field->basename . "'] =  note_" . $field->basename . ";\n";

        # asset_ids
        $asset_ids = "var asset_ids = new Array();\nvar asset_ids_val = '';\njQuery('.include_asset_ids-' + id).each(function(){\nasset_ids[asset_ids.length] = jQuery(this).val();\n});\n
if(asset_ids.length > 1){\nasset_ids_val = '[' + asset_ids.join(',') + ']';\n} else if(asset_ids.length == 1){\nasset_ids_val = asset_ids[0];\n} else {\nasset_ids_val = 0;\n}\n";

        if ($field->type eq 'image') {
            $old_image_buf .= "var old_image_" . $field->basename . " = '';\n";
            $image_buf .= "old_image_" . $field->basename . " = jQuery('#image-" . $field->basename . "-' + id).html();\n";
            $recover_image_buf .= "if(old_note_" . $field->basename . " == '---') {\nold_note_" . $field->basename . " = '';\n}\njQuery('#image-" . $field->basename . "-' + id).html( old_image_" . $field->basename . " );\nif (old_image_" . $field->basename . " == '---'){\n  jQuery('#delete-" . $field->basename . "-' + id).css('display', 'none');\n} else {\n  jQuery('#delete-" . $field->basename . "-' + id).css('display', 'inline');\n}\n";
            $clear_image_buf .=  "old_image_" . $field->basename . " = '';\n";
        }

        $counter++;
    }
    return 1 if !$counter;

### copy for category version
    if (!$asset_ids) {
        $asset_ids = "asset_ids_val = ''";
    }
###

    my $cancel = join(' && ', @cancel_buf);

    my $plugin = $app->component('PowerListingFieldEditor');
    my $field = $tmpl->getElementById('container');
    my $node = $tmpl->createElement('setvarblock', { name => 'jq_js_include', component => $plugin });

    my $entry_id_message = $plugin->translate('entry_id');
    my $update_message = $plugin->translate('update...');
    my $end_message = $plugin->translate('done update');

    my $html = <<HTML;
    var current_edit = 0;
    $old_buf
    $old_image_buf

    function clear_image(id, basename_id) {
        basename_id.match(/delete-(.*)/);
        var basename_id = RegExp.\$1;
        jQuery('#image-' + basename_id).html('---');
        jQuery('#note-' + basename_id).val('');
        jQuery('#include_asset_ids-' + basename_id).val('');
        jQuery('#delete-' + basename_id).css('display', 'none');
    }

    function edit_note(id) {
        if (current_edit) return;
        var note_links = new Array();
        $link_buf
        var note_fields = new Array();
        $field_buf
        if (!note_links.length || !note_fields.length) return;
        $note_buf
        current_edit = id;
        $old_note_buf
        $image_buf
        $empty_note_buf

        for ( i = 0; i < $counter; ++i ) {
            note_links[i].hide();
            note_fields[i].show();
        }
        return false;
    }

    function cancel_edit(id) {
        var note_links = new Array();
        $link_buf
        var note_fields = new Array();
        $field_buf
        $note_buf
        if (!note_links.length || !note_fields.length) return;
        $recover_image_buf
        $recover_buf
        $clear_image_buf
        $clear_note_buf
        current_edit = 0;

        for ( i = 0; i < $counter; ++i ) {
            note_links[i].show();
            note_fields[i].hide();
        }
        return false;
    }

    function submit_edit(id) {
        if ((0 != current_edit) && (id != current_edit))
            id = current_edit;
        $note_buf
        $new_note_buf

        if ($cancel) {
            cancel_edit(id);
            return false;
        }
        return submit_edit_form(id);
    }

    function submit_edit_form(id) {
        $submit_buf
        $asset_ids
        renderList({
            __mode: 'save_from_list',
            _type: '$type',
            customfield_beacon: 1,
            $render_buf
            include_asset_ids: asset_ids_val,
            id: id,
            forward_list: 1,
            magic_token: 1
        });
        cancelCreateMode();
        return false;
    }

    function setCreateMode () {
        jQuery('#create-inline-notification').show();
        jQuery('#text').val('').focus();
        jQuery('#content-actions').hide();
    }

    function cancelCreateMode () {
        jQuery('#create-inline-notification').hide();
        jQuery('#create-inline-notification input').mtUnvalidate();
//        jQuery('#content-actions').show();

        current_edit = 0;
    }

    jQuery('.start-edit').live('click', function () {
        jQuery(this).attr('href').match(/#(\\d+)/);
        var id = RegExp.\$1;
        cancel_edit(current_edit);
        edit_note(id);
        return false;
    });

    jQuery('.cancel-edit').live('click', function () {
        jQuery(this).attr('href').match(/#(\\d+)/);
        var id = RegExp.\$1;
        cancel_edit(id);
        return false;
    });

    jQuery('.submit-note').live('click', function () {
        jQuery(this).attr('href').match(/#(\\d+)/);
        var id = RegExp.\$1;
        submit_edit(id);
        return false;
    });

    jQuery(window).bind('listReady', function() {
        jQuery('a.mt-open-dialog').mtDialog();

        jQuery('.clear-image').each(function () {
            jQuery(this).attr('href').match(/#(\\d+)/);
            var id = RegExp.\$1;
            jQuery(this).attr('id').match(/delete-(.*)/);
            var basename_id = RegExp.\$1;
            if (jQuery('#image-' + basename_id).html() == '---'){
                jQuery(this).css('display', 'none');
            } else {
                jQuery(this).css('display', 'inline');
            }
        });
    });

    jQuery('.clear-image').live('click', function () {
        jQuery(this).attr('href').match(/#(\\d+)/);
        var id = RegExp.\$1;
        var basename = jQuery(this).attr('id');
        clear_image(id, basename);
        return false;
    });

    jQuery('#uploadStatePnl').hide();
    jQuery(window).bind('listReady', function() {
//alert("OK");
// remove checkbox
jQuery('.col.head.cb.first-visible-child.first-child input').removeAttr('checked');
jQuery('#entry-table tr').removeClass('selected');
jQuery('#page-table tr').removeClass('selected');

        // end save
        if (done) {
            incr();
            updateStatus(last_id + 1);
            done = 0;
jQuery('#create-inline-notification').hide();
jQuery('#create-inline-notification input').mtUnvalidate();

            jQuery('#uploadStatePnl').animate({
                opacity: 'toggle',}, { duration: 2000, easing: 'swing',
                complete: function () {
                    jQuery("#content-actions").show();
                    jQuery("#display-options").show();

        jQuery("#menu").show();
        jQuery(".has-menu-nav #content").css('margin-left', '175px');
        jQuery("#sidebar_none a").show();
        jQuery("#sidebar_block a").hide();

                }
            });
        // 一括編雁EにフィールドE増渁E
        } else {
            if(jQuery('#create-inline-notification').css('display') == 'none') {
                jQuery("#content-actions").show();
            }
            if(jQuery('#create-inline-notification').css('display') == 'block') {
                setEditAllMode();
            }
        }
    });

    jQuery("#progressbar").progressbar();
    jQuery('#submit-all-field').click( function() {
        submit_all_edit();
        return false;
    });

    jQuery('#action-edit-all').click( function() {
        jQuery("#content-actions").hide();
//        jQuery("#display-options").hide();
        setEditAllMode();
        return false;
    });

    jQuery('#create-inline-notification button.mt-cancel-action').click(function() {
//        jQuery("#display-options").show();
        cancelEditAllMode();
        return false;
    });

    jQuery('#sidebar_none a').click(function(){
        jQuery(".mt-primaryNavigation").hide();

        jQuery("#sidebar_none a").hide();
        jQuery("#sidebar_block a").show();
    });
    jQuery('#sidebar_block a').click(function(){
        jQuery(".mt-primaryNavigation").show();

        jQuery("#sidebar_none a").show();
        jQuery("#sidebar_block a").hide();
    });

    function cancelEditAllMode () {
        jQuery('#entry-table input[type=checkbox]').each(function() {
var id = this.id;
id.match(/select_(\\d+)/);
var data = RegExp.\$1;
            cancel_edit_all(data);

            jQuery('.buttons', jQuery(this).parent().parent().parent()).each(function() {
                jQuery(this).show();
            });
        });
        jQuery('#create-inline-notification').hide();
        jQuery('#content-actions').show();

        jQuery("#menu").show();
        jQuery(".has-menu-nav #content").css('margin-left', '175px');
        jQuery("#sidebar_none a").show();
        jQuery("#sidebar_block a").hide();

        current_edit = 0;
    }

    function setEditAllMode () {
        jQuery('#create-inline-notification').show();
        jQuery('#content-actions').hide();

        cancel_edit(current_edit);

        $all_note_buf_new
        $all_old_note_buf_new
        $all_check_note_buf_new

        jQuery('#entry-table input[type=checkbox]').each(function() {
var id = this.id;
id.match(/select_(\\d+)/);
var data = RegExp.\$1;
            edit_all_note(data);
            jQuery('.buttons', jQuery(this).parent().parent().parent()).each(function() {
                jQuery(this).hide();
            });
        });
        current_edit = 'all';
    }

    function edit_all_note(id) {
        var note_links = new Array();
        $link_buf
        var note_fields = new Array();
        $field_buf
        if (!note_links.length || !note_fields.length) return;
        $all_note_buf
        $all_old_note_buf
        $all_image_buf
        $all_empty_note_buf

        for ( i = 0; i < $counter; ++i ) {
            note_links[i].hide();
            note_fields[i].show();
        }
        return false;
    }

    function cancel_edit_all(id) {
        var note_links = new Array();
        $link_buf
        var note_fields = new Array();
        $field_buf
        $all_note_buf
        if (!note_links.length || !note_fields.length) return;
//        $all_recover_image_buf
//        $all_recover_buf
//        $all_clear_image_buf
//        $all_clear_note_buf
        current_edit = 0;

        for ( i = 0; i < $counter; ++i ) {
            note_links[i].show();
            note_fields[i].hide();
        }
        return false;
    }

    var PageState = {
        SELECT: 0,
        UPLOADING: 1
    };
    var total;
    var done = 0;
    var beginTime;
    var incr = function(){
        ++done;
    };
    var isAbort = false;
    var abort = function(){
        isAbort = true;
        updatePage(PageState.SELECT);
        alert("stopped!");
    };
    var updatePage = function(mode){
        switch(mode){
            case PageState.SELECT:
                jQuery("#content-actions").show();
                jQuery("#uploadStatePnl").hide();
                jQuery("#display-options").show();
                break;
            case PageState.UPLOADING:
                jQuery("#display-options").hide();
                jQuery("#content-actions").hide();
                jQuery("#filePreviewPnl").hide();
                jQuery("#uploadStatePnl").show();
                updateProgress(0);
                break;
        }
    }
    var updateProgress = function(value){
        jQuery("#progressbar").progressbar("value", value);
    }
    var updateStatus = function(id){
        updateProgress(done / total * 100);

        var exec_file = id;
        if (done < total) {
            exec_file = id;
            jQuery("#progressmsg").text("$entry_id_message" + id + '$update_message' + " ( " + done + " / " + total + " $end_message )");
        } else if (done == total) {
            jQuery("#progressmsg").text(" $end_message(" + done + " / " + total + ")");
        }
    };

    var last_id = 0;
    var submit_all_edit = function() {

        var i = 0;
        var list = new Array();
        jQuery('#entry-table input[type=checkbox]').each(function() {
            var id = this.id;
            if (id.match(/select_(\\d+)/)) {
                var data = RegExp.\$1;
                list.push(data);
            }
        });
        var len = list.length;
        total = list.length;
        var retry = 0;
        var iter = function(){
                var id = list[i++];
                $all_submit_buf
                $asset_ids

                var param = {};
                param['__mode'] = 'save_from_list';
                param['_type'] = 'entry';
                param['blog_id'] = $blog_id;
                param['customfield_beacon'] = 1;
                $all_render_buf
                param['include_asset_ids'] = asset_ids_val; // overwrite note_include_asset_ids
                param['id'] = id;
                param['forward_list'] = 1;
                param['magic_token'] = 1;

                jQuery.ajax({
                    url: script_url,
                    type: "POST",
                    data: param,
                    success: function() {
                        if (isAbort) {
                            return;
                        }
                        if (i < len) {
                            incr();
                            updateStatus(list[i]);
                            retry = 0;
                            iter();
                        } else {
                            incr();
                            updateStatus(id);
                            listStore.trigger('load_list');
                        }
                    },
                    error: function(xhr, textStatus, errorThrown) {
                        if (isAbort) {
                            return;
                        }
                        console.log(xhr);
                        console.log(textStatus + ": " + errorThrown);

                        if (xhr.status == 500 && retry < 3) {
                            console.log("retry: " + this.id);

                            i--;
                            retry++;
                            iter();
                        } else {
                            alert( this.id + "is error");
                            if(confirm("continue?")){
                                iter();
                            } else {
                                abort();
                            }
                        }
                    }
                });
        };

        updateStatus(list[i]);

        jQuery('.indicator, #listing-table-overlay').show();
        jQuery('div.pagination').hide();
        jQuery('#msg-block div.msg-error.filtered-list-msg').remove();
jQuery('#create-inline-notification').hide();
        updatePage(PageState.UPLOADING);
        iter();

//        cancelCreateMode();
        return false;
    };

var cols = [];
var vals = [];
var checked = [];
var total = 0;
var editable_total = 0;
var editable_count_in_page;
var currentPage = 1;

var request_id = 0;
function renderList(mode, columns, values, limit, page ) {
  var this_request = ++request_id;

var params = [];
jQuery('.custom-control.custom-checkbox input[type=checkbox]:checked').each(function(){
//    if (this.id.match(/select/)) {
//    } else {
    params.push(this.id);
//    }
});

  // If the first arg is Object, it's Hash style call
  var args = {};
  if ( typeof mode == 'object' ) {
    args = mode;
    var defaults = {
      mode: 'filtered_list',
      columns: listStore.columns.join(','),
      values:  vals,
      limit: listStore.limit,
      page: listStore.page
    };
    mode    = ( 'mode' in args )    ? args['mode']    : defaults['mode'];
    columns = listStore.columns.join(',');
    values  = ( 'values' in args )  ? args['values']  : defaults['values'];
    limit   = listStore.limit;
    page    = listStore.page;
    delete args['mode'];
    delete args['columns'];
    delete args['values'];
    delete args['limit'];
    delete args['page'];
  }
  limit = parseInt(limit);
  if ( args['magic_token'] ) {
    args['magic_token'] = jQuery('input[name="magic_token"]').val();
  }
  var params = {
    __mode: mode,
    datasource: '<mt:var name="list_type">',
    blog_id: <mt:var name="blog_id">,
    columns: params.join(','),
    limit: listStore.limit,
    page: listStore.page,
    magic_token: listStore.listClient.magicToken,
//items: [],
sort_by: 'authored_on',
sort_order: 'descend',
fid: '_allpass'

  };
  params = jQuery.extend( params, args );

  if (values.length) {
    params['items'] = jQuery.toJSON(values);
  }
  if (jQuery('table.mt-table th.sorted').length) {
    var sorted = jQuery('table.mt-table th.sorted').attr('data-id');
    params['sort_by'] = sorted;
    params['sort_order'] = jQuery('table.mt-table th.sorted').find('span').hasClass('desc') ? 'descend' : 'ascend';
  }
  if (jQuery('input[name=filter_id]').val()) {
    params['fid'] = jQuery('input[name=filter_id]').val();
  }
  if (jQuery('input[name=filter_name]').val()) {
    params['label'] = jQuery('input[name=filter_name]').val();
  }

  jQuery('.indicator, #listing-table-overlay').show();
  jQuery('div.pagination').hide();
  jQuery('#msg-block div.msg-error.filtered-list-msg').remove()
  // adding random bits to avoid cache in Android.
  var script_url = '<mt:var name="script_url">?.r=' + Math.random(1);
  jQuery.ajax({
    type: 'POST',
    contentType: 'application/x-www-form-urlencoded; charset=utf-8',
    url: script_url,
    dataType: 'json',
    data: params,
    success: function(data) {
      if ( this_request != request_id ) {
          return false;
      }

      if ( !handleMessages(data, 'filtered-list-msg' ) ) return;
<mt:if name="config.debugmode">
      if (data.result.debug) {
        jQuery('#listing-debug-block').text(data.result.debug);
      }
</mt:if>
      jQuery('table.mt-table tbody').empty();
      jQuery('table.listing-table th :checkbox').prop('checked',false);
      var objs = data.result.objects;
      var count = objs.length;
//console.log("count:"+count);
      var last = data.result.page_max;
      total = parseInt(data.result.count);
      editable_total = parseInt(data.result.editable_count);
      var return_columns = data.result.columns.split(',');
//console.log("return_columns:"+return_columns);
      var return_cols = jQuery.grep( return_columns, function( val, idx ){
        return val.match(/\./);
      }, true );
      var main_cols = {};
      jQuery('#disp_cols input.main.checkbox:checked').each(function() {
        main_cols[ jQuery(this).val() ] = 1;
      });
      editable_count_in_page = 0;
      for (var row = 0; row < count; row++) {
//console.log("row:"+row);
        var id = objs[row][0];
        var line = '';
        if ( id ) {
          editable_count_in_page++;
          if (checked[page] && checked[page][id]) {
//            line = '<td class="cb col"<mt:unless name="has_list_actions"> style="display: none;"</mt:unless>><input type="checkbox" checked="checked" name="id" value="'+id+'" /></td>';
              line = '<td><div class="custom-control custom-checkbox"><input type="checkbox" name="id" class="custom-control-input" id="select_' + (row+1) + ' value="' + id + '"><span class="custom-control-indicator"></span><label class="custom-control-label" for="select_' + (row+1) + '"><span class="sr-only"><__trans phrase="Select" escape="js"></span></label></div></td>';
          } else {
//            line = '<td class="cb col"<mt:unless name="has_list_actions"> style="display: none;"</mt:unless>><input type="checkbox" name="id" value="'+id+'" /></td>';
              line = '<td><div class="custom-control custom-checkbox"><input type="checkbox" name="id" class="custom-control-input" id="select_' + (row+1) + ' value="' + id + '"><span class="custom-control-indicator"></span><label class="custom-control-label" for="select_' + (row+1) + '"><span class="sr-only"><__trans phrase="Select" escape="js"></span></label></div></td>';
          }
        }
        else {
          line = '<td class="cb col"<mt:unless name="has_list_actions"> style="display: none;"</mt:unless>></td>';
        }
        for (var col = 1, col_length = objs[row].length; col < col_length; col++) {
//console.log("col:"+col+ " data:"+objs[row][col]);
          var col_id = return_cols[ col - 1 ];
//          if ( main_cols[col_id] )
//              line += '<td class="col '+col_id+' '+'">' + objs[row][col] + '</td>';
// contentɒlƃCAEgꂪ
              line += '<td data-is="list-table-column" class="' + col_id + '" content="' + '' + '">' + objs[row][col] + '</td>';
        }
        jQuery('<tr data-is="list-table-row" data-index="' + row + '"' + ( id ? '' : ' class="no-action"' ) + '>'+line+'</tr>').appendTo('table.mt-table tbody');
      }
      jQuery('#resultstats').remove();
      var n = columnLength();
      if (count == 0) {
        var message = trans('No [_1] could be found.', '<mt:if name="zero_state"><mt:var name="zero_state" escape="js"><mt:else><mt:var name="object_label" lower_case="1" escape="js"></mt:if>');
        jQuery('<td class="col fullwidth-row" colspan="'+n+'">'+message+'</tr>').appendTo('table.mt-table tbody');
      }
//      renderPagination(count, limit, page, last, total);
      filters = data.result.filters;
      if ( data.result.label ) {
        jQuery('#opener').html(data.result.label);
        jQuery('#filter_name').val(data.result.label);
      }
      if ( data.result.id ) {
        jQuery('#filter_id').val(data.result.id);
        current_id = data.result.id;
      }
    },
    complete: function() {

//listStore.resetColumns();
//          riot
//          self.updateIsLoading(false);
//          if (refreshCurrentFilter) {
//            self.trigger('refresh_current_filter')
//          }
//          self.trigger('refresh_view', { moveToPagination: moveToPagination });

//      if ( this_request != request_id ) {
//          return false;
//      }
//      jQuery('.indicator, #listing-table-overlay').hide();
//      jQuery('div.pagination').show();

//      jQuery('table.listing-table tbody tr:even').addClass('even');
//      jQuery('table.listing-table tbody tr:odd').addClass('odd');
//      var \$checkboxes = jQuery('td :checkbox');
//      var \$checked = jQuery('td :checked');
//      \$checked.parents('tr').addClass('selected');
//      if (\$checkboxes.length && \$checkboxes.length == \$checked.length) {
//        jQuery('table.listing-table th :checkbox').prop('checked', true);
//        addSelectAll(\$checkboxes);
//      }
//      if (mode == 'save_filter' || mode == 'delete_filter') {
//        renderFilterList();
//        updateFilterSaveButtons({
//          system:     0,
//          new_filter: 0
//        });
//      }

//      if (!jQuery('#msg-block .msg-error').length) {
//        jQuery('#dialog_save_filter').dialog('close');
//      } else {
//        if (jQuery('#dialog_save_filter:visible').length) {
//          jQuery('.msg-error').clone().appendTo('#dialog-msg-block');
//          jQuery('#msg-block').hide();
//        }
//      }
//      jQuery(window).trigger('listReady');



    },
    error: function(xhr, status, error) {
      if ( this_request != request_id ) {
          return false;
      }
      if ( xhr.status == 401 ) {
        loginAgain(function(){
          renderList(mode, columns, values, limit, page);
        });
      }
      else if ( xhr.status == 0 ) {
        // Maybe this is user abort. do nothing.
      }
      else {
        alert( trans('Communication Error (HTTP status code: [_1]. Message: [_2])', xhr.status, error) );
      }
    }
  });
}

function columnLength () {
  return jQuery('#disp_cols').find('input.main:checked').length<mt:if name="has_list_actions"> + 1</mt:if>;
}

function renderPagination(count, limit, page, last, total) {
  var \$e = jQuery('.pagination').empty();
  if (count == 0) {
    return;
  }
  var usefirst = false;
  var uselast = false;
  var start;
  if (page > 1) {
    usefirst = true;
    start = (page-1)*limit+1;
  } else {
    start = 1;
  }
  if (total / limit > 1) {
    if (page + 1 <= last) {
      uselast = true;
    }
  }
  currentPage = page;

  if (usefirst) {
    \$e.append('<a href="#" class="pagenav start">&laquo; <__trans phrase="First" escape="js"></a>')
      .append('<a href="#" class="pagenav to-start">&lsaquo; <__trans phrase="Prev" escape="js"></a>');
  } else {
    \$e.append('<span class="pagenav start disabled">&laquo; <__trans phrase="First" escape="js"></span>')
      .append('<span class="pagenav to-start disabled">&lsaquo; <__trans phrase="Prev" escape="js"></span>');
  }
  var end = (page == last) ? total : start+limit-1;
  var page_status = trans('[_1] - [_2] of [_3]', start, end, total);

  \$e.append('<span class="current-rows">'+page_status+'</span>')
  if (uselast) {
    \$e.append('<a href="#" class="pagenav to-end"><__trans phrase="Next" escape="js"> &rsaquo;</a>')
      .append('<a href="#" class="pagenav end"><__trans phrase="Last" escape="js"> &raquo;</a>');
  } else {
    \$e.append('<span class="pagenav to-end disabled"><__trans phrase="Next" escape="js"> &rsaquo;</span>')
      .append('<span class="pagenav end disabled"><__trans phrase="Last" escape="js"> &raquo;</span>');
  }
  jQuery('a.start').click(function() {
    saveChecked(page);
    renderList('filtered_list', cols, vals, jQuery('#row').val(), 1);
    return false;
  });
  jQuery('a.to-start').click(function() {
    saveChecked(page);
    renderList('filtered_list', cols, vals, jQuery('#row').val(), page-1);
    return false;
  });
  jQuery('a.to-end').click(function() {
    saveChecked(page);
    renderList('filtered_list', cols, vals, jQuery('#row').val(), page+1);
    return false;
  });
  jQuery('a.end').click(function() {
    saveChecked(page);
    renderList('filtered_list', cols, vals, jQuery('#row').val(), last);
    return false;
  });
}

HTML

    $node->innerHTML($html);
    $tmpl->insertBefore($node, $field);

#    my $plugin = MT->component("PowerListingFieldEditor");
#    my $node2 = $tmpl->createElement('setvarblock', { name => 'content_header', append => 1 });
#    my $html2 = 'Can use ' . rest_day() . 'days';
#    $node2->innerHTML($html2);
#    $tmpl->insertBefore($node2, $field);

    1;
}

sub add_css {
    my ($cb, $app, $template) = @_;
    return if is_limit();

    my $q = $app->param;
    my $mode = $q->param('__mode');
    return if !($mode eq 'list' || $mode eq 'filtered_list' || $mode eq 'save_from_list');

    my $type = $q->param('_type');
    return if !($type eq 'entry' || $type eq 'page' || $type eq 'comment' || $type eq 'asset' || $type eq 'member' || $type eq 'content_data');

    my $blog_id = $app->param('blog_id') || 0;

    my $asset_type = [qw( file audio image video)] if $type eq 'asset';

    my @fields = CustomFields::Field->load({
        obj_type => ($asset_type) ? $asset_type : $type eq 'member' ? 'author' : $type,
        blog_id => $blog_id ? [ $blog_id, 0 ] : 0,
    });
#    return if !@fields;

    my $plugin = $app->component('PowerListingFieldEditor');
    my $text_width = $plugin->get_config_value('text_width', 'blog:'.$app->param('blog_id')) || 250;
    my $textarea_width = $plugin->get_config_value('textarea_width', 'blog:'.$app->param('blog_id')) || 400;
    my $text = $text_width - 15;
    my $textarea = $textarea_width - 20;

    my @text_selectors;
    my @textarea_selectors;
    my @button_selectors;
    my @button_selectors2;
    foreach my $field (@fields) {
        next if !( $field->type eq 'text' ||
                   $field->type eq 'textarea' ||
                   $field->type eq 'checkbox' ||
                   $field->type eq 'select' ||
                   $field->type eq 'radio' ||
                   $field->type eq 'url' ||
                   $field->type eq 'image' );
        if ( $field->type eq 'text' || $field->type eq 'url' ) {
            push @text_selectors, ".col.head.blog" . $blog_id . "_" . $field->basename;
        } elsif ( $field->type eq 'textarea' ) {
            push @textarea_selectors, ".col.head.blog" . $blog_id . "_" . $field->basename;
        } elsif ( $field->type eq 'checkbox' || $field->type eq 'radio' || $field->type eq 'select' || $field->type eq 'image' ) {
            push @button_selectors, ".col.head.blog" . $blog_id . "_" . $field->basename;
            if ( $field->type eq 'radio' ) {
            push @button_selectors2, "td.col.blog" . $blog_id . "_" . $field->basename . " .buttons";
            }
        }
    }

    my $title_selector;
    for my $col (qw( title text excerpt text_more keywords include_asset_ids tags )) {
        if ( $col =~ /text|excerpt/ ) {
            push @textarea_selectors, ".col.head.blog" . $blog_id . "_" . $col;
        } else {
            push @text_selectors, ".col.head.blog" . $blog_id . "_" . $col;
        }
        if ( $col eq 'title' ) {
            $title_selector = ".col.head.blog" . $blog_id . "_" . $col . '{ width: auto; min-width: 30px; }' .
                              ".col.blog" . $blog_id . "_" . $col . ' input { width: 100%; }';
        }
    }

    my $date_selector;
    for my $col (qw( authored_on modified_on )) {
        $date_selector .= ".col.head.blog" . $blog_id . "_" . $col . '{ /* max-width: 150px; width: 150px; */ }';
    }

    push @button_selectors, ".col.head.blog" . $blog_id . "_" . 'include_asset_ids';
    push @button_selectors, ".col.head." . 'category2';
    push @button_selectors, ".col.head." . 'status3';

#    my $text_selector = join(",", @text_selectors) . "{width: ${text_width}px;}" if @text_selectors;
    my $text_selector = join(",", @text_selectors) . "{width: 18%;}" if @text_selectors;
#    my $textarea_selector = join(",", @textarea_selectors) . "{width: ${textarea_width}px;}" if @textarea_selectors;
    my $textarea_selector = join(",", @textarea_selectors) . "{width: 30%;}" if @textarea_selectors;

#    my $button_selector = join(",", @button_selectors) . "{width: 140px;}" if @button_selectors;
    my $button_selector = join(",", @button_selectors) . "{width: 10%;}" if @button_selectors;
    my $button_selector2 = join(",", @button_selectors2) . "{display: block; margin-top: 3px;}" if @button_selectors2;

    my $old;
    my $new;

    $old = <<HTML;
<mt:var name="html_head">
HTML
    $old = quotemeta($old);
    $new = <<HTML;
<style type="text/css">
$date_selector
$text_selector
$textarea_selector
$button_selector
$button_selector2

$title_selector

a.cancel-edit, a.submit-note {
    padding: 3px 8px;
    text-decoration: none;
    color: #000;
}

.lfetext, .lfetextarea, .lfeurl, .lfedate {
    -moz-border-bottom-colors: none;
    -moz-border-image: none;
    -moz-border-left-colors: none;
    -moz-border-right-colors: none;
    -moz-border-top-colors: none;
    -moz-box-sizing: border-box;
    background-color: #FBFBFB;
    border-color: #ADADAD #DCDDDD #DCDDDD #ADADAD;
    border-radius: 3px 3px 3px 3px;
    border-right: 1px solid #DCDDDD;
    border-style: solid;
    border-width: 1px;
    box-shadow: 0 1px 0 #F3F3F3;
    font-size: 13px;
    outline: 0 none;
    padding: 0.2em 0.25em;
    vertical-align: middle;
    margin-bottom: 3px;
}
.lfetext, .lfeurl {
/*    width: ${text}px; */
    width: 100%;
}
.lfedate {
/*
    width: 135px;
    max-width: 135px;
*/
    width: 100%; 
}
.lfetextarea {
    vertical-align: bottom;
    height: 100px;
/*    width: ${textarea}px; */
    width: 100%; 
}
.lfecheckbox {
    margin-bottom: 3px;
}
.lfeselect {
    margin-bottom: 3px;
}
.lferadio .submit-note, .lferadio .cancel-note {
    margin-bottom: 3px;
}
.image-link {
    margin: 3px 0;
}
</style>
<script>
        var script_url = '<mt:var name="script_url">';
        var blog_id = '<mt:var name="blog_id">';
        var magic_token = "<mt:var name="magic_token">";
</script>
  <mt:var name="html_head">
HTML

    $$template =~ s/$old/$new/;
}

sub delete_js {
    my ($cb, $app, $template) = @_;

    my $q = $app->param;
    my $mode = $q->param('__mode');
    my $type = $q->param('_type');
    my $blog_id = $q->param('blog_id') || 0;

    return if !$blog_id;
    return if $mode ne 'list';
    return if $type ne 'entry' && $type ne 'page' && $type ne 'content_data';

    my $old;
    my $new;

    $old = <<HTML;
<script type="text/javascript" src="<mt:var name="static_uri">js/listing/list_data.js?v=<mt:var name="mt_version_id" escape="url">"></script>
HTML
    $old = quotemeta($old);
    $new = <<HTML;
<script type="text/javascript" src="<mt:var name="static_uri">plugins/PowerListingFieldEditor/js/list_data.js?v=<mt:var name="mt_version_id" escape="url">"></script>
HTML
    $$template =~ s/$old/$new/;

    $old = <<HTML;
  <script type="text/javascript" src="<mt:var name="static_uri">js/listing/listing.js?v=<mt:var name="mt_version_id" escape="url">"></script>
HTML
    $old = quotemeta($old);
    $new = <<HTML;
  <script type="text/javascript" src="<mt:var name="static_uri">js/listing/listing.js?v=<mt:var name="mt_version_id" escape="url">"></script>
  <script type="text/javascript" src="<mt:var name="static_uri">plugins/PowerListingFieldEditor/js/add.js?v=<mt:var name="mt_version_id" escape="url">"></script>
  <script type="text/javascript" src="<mt:var name="static_uri">js/editor.js"></script>
HTML
    $$template =~ s/$old/$new/;

    my $plugin = $app->component("PowerListingFieldEditor");
    my $display = $plugin->translate('Display');
    my $hidden = $plugin->translate('Hidden');

    $old = <<HTML;
<mt:setvarblock name="system_msg" append="1">
<div id="msg-block"></div>
</mt:setvarblock>
HTML
    $old = quotemeta($old);
    $new = <<HTML;
<mt:setvarblock name="system_msg" append="1">
<div id="msg-block"></div>
<div style="margin-top:20px">
<mt:var name="rest_day">
</div>
<link rel="stylesheet" type="text/css" href="<mt:var name="static_uri">plugins/PowerListingFieldEditor/css/jquery-ui-1.8.17.custom.css" />
<form class="create-inline" id="create-inline-notification" style="display: none;">
    <span class="buttons">
      <button
         type="submit"
         id="submit-all-field"
         class="button">
        <__trans phrase="Save">
      </button>
      <button
         type="button"
         class="button mt-cancel-action">
        <__trans phrase="Cancel">
      </button> 
    </span>
<span id="sidebar_none"><a href="javascript:void(0)">$hidden</a></span>
<span id="sidebar_block"><a href="javascript:void(0)" style="display:none">$display</a></span>
</form>
</mt:setvarblock>
<mt:setvarblock name="content_header" append="1">
<div id="uploadStatePnl">
    <p id="start_message" style="margin-bottom:5px"><mt:var name="uploading" /></p>
    <div id="progressbar"></div>
    <p id="progressmsg"></p>
</div>
</mt:setvarblock>
HTML
    $$template =~ s/$old/$new/;
}

########################################################

sub add_param {
    my ($cb, $app, $param, $tmpl) = @_;

    my $plugin = MT->component("PowerListingFieldEditor");
    $param->{rest_day} = $plugin->translate('can use [_1]', rest_day());

    1;
}

sub is_limit {
    return (rest_day() > 0) ? 0 : 1;
}

sub rest_day {
    my $plugin = MT->component("PowerListingFieldEditor");
    my $field_column = $plugin->get_config_value('field_column', 'system') || '';
    my $duration = time() - $field_column;

    my $rest_day = 31 - int($duration / (24 * 3600));

    return ($rest_day > 0) ? $rest_day : 0;
}

sub save_from_list {
    my $app = shift;

    return $app->forward('filtered_list') if is_limit();
#    $app->validate_magic or return;
    my $author = $app->user;
    my $type = $app->param('_type') || 'entry';
    $type = 'author' if $type eq 'member';

    my $class = $app->model($type)
        or return $app->errtrans("Invalid parameter");

    my $perms;
    if ( $type eq 'entry' || $type eq 'page' ) {
        $perms = $app->permissions
            or return $app->permission_denied();
    }

    my $id = $app->param('id');
    return if !$id;
    if ( $type eq 'page' ) {
        return $app->permission_denied()
            unless $perms->can_do('edit_all_pages');
    }

    require MT::Blog;
    my $blog_id = $app->param('blog_id');
    my $blog    = MT::Blog->load($blog_id)
        or return $app->error(
        $app->translate( 'Can\'t load blog #[_1].', $blog_id ) );
    my $obj = $class->load($id)
        || return $app->error(
        $app->translate( "No such [_1].", $class->class_label ) );

    if ($type ne 'author') {
        return $app->error( $app->translate("Invalid parameter") )
            unless $obj->blog_id == $blog_id;
    }

    if ( $type eq 'entry' ) {
        return $app->permission_denied()
            unless $perms->can_edit_entry( $obj, $author );
    }

    foreach ( $app->param() ) {
        next if $_ =~ /customfield_/;
        if ($_ eq 'title' || $_ eq 'text' || $_ eq 'text_more' || $_ eq 'excerpt' || $_ eq 'keywords' || $_ eq 'status') {
            $obj->$_($app->param($_));
        } elsif ($_ eq 'authored_on' || $_ eq 'modified_on') {
            (my $ts = $app->param($_)) =~ s/(\d{4})-(\d{2})-(\d{2})\s(\d{2}):(\d{2}):(\d{2})/$1$2$3$4$5$6/;
            $obj->$_($ts) if $ts =~ /^\d{14}$/;
        } elsif ($_ eq 'tags') {
            my $tags = $app->param($_);
            if ( defined $tags ) {
                my $blog   = $app->blog;
                my $fields = $blog->smart_replace_fields;
                if ( $fields =~ m/tags/ig ) {
                    $tags = MT::App::CMS::_convert_word_chars( $app, $tags );
                }

                require MT::Tag;
                my $tag_delim = chr( $app->user->entry_prefs->{tag_delim} );
                my @tags = MT::Tag->split( $tag_delim, $tags );
                if (@tags) {
                    $obj->set_tags(@tags);
                }
                else {
                    $obj->remove_tags();
                }
            }
        } elsif ($_ eq 'category') {
            my $cat_id = $app->param($_);
            my $place  = MT::Placement->load(
                {   entry_id   => $id,
                    is_primary => 1
                }
            );
            if ( $place && !$cat_id ) {
                $place->remove
                    or return $app->error(
                    $app->translate(
                        "Removing placement failed: [_1]",
                        $place->errstr
                    )
                    );
            } elsif ($cat_id) {
                unless ($place) {
                    $place = MT::Placement->new;
                    $place->entry_id($id);
                    $place->blog_id($blog_id);
                    $place->is_primary(1);
                }
                $place->category_id($cat_id);
                $place->save
                    or return $app->error(
                    $app->translate(
                        "Saving placement failed: [_1]",
                        $place->errstr
                    )
                    );
            }
        }
    }
    $obj->modified_by( $author->id );
    $obj->save;

    CustomFields::App::CMS::CMSPostSave_customfield_objs('dummy', $app, $obj);

    # for asset
    require MT::Asset;
    require MT::ObjectAsset;
    my $include_asset_ids = $app->param('include_asset_ids');

    if ($include_asset_ids) {
        my @asset_ids;
        if ($include_asset_ids =~ /\[(.*)\]/) {
            @asset_ids = split( ',', $1 );
        } else {
            push @asset_ids, $include_asset_ids;
        }

        my $obj_assets        = ();
        my @obj_assets        = MT::ObjectAsset->load(
            { object_ds => 'entry', object_id => $obj->id } );
        foreach my $obj_asset (@obj_assets) {
            my $asset_id = $obj_asset->asset_id;
            $obj_assets->{$asset_id} = 1;
        }
        my $seen = ();
        foreach my $asset_id (@asset_ids) {
            my $obj_asset = MT::ObjectAsset->load(
                {   asset_id  => $asset_id,
                    object_ds => 'entry',
                    object_id => $obj->id
                }
            );
            unless ($obj_asset) {
                my $obj_asset = new MT::ObjectAsset;
                $obj_asset->blog_id($blog_id);
                $obj_asset->asset_id($asset_id);
                $obj_asset->object_ds('entry');
                $obj_asset->object_id( $obj->id );
                $obj_asset->save;
            }
            $seen->{$asset_id} = 1;
        }
        foreach my $asset_id ( keys %{$obj_assets} ) {
            unless ( $seen->{$asset_id} ) {
                my $obj_asset = MT::ObjectAsset->load(
                    {   asset_id  => $asset_id,
                        object_ds => 'entry',
                        object_id => $obj->id
                    }
                );
                $obj_asset->remove;
            }
        }
    } else {
        my @obj_assets = MT::ObjectAsset->load(
            { object_ds => 'entry', object_id => $obj->id } );
        return $app->forward('filtered_list') if !@obj_assets;
        foreach my $obj_asset ( @obj_assets ) {
            $obj_asset->remove;
        }
    }
    return $app->forward('filtered_list');
}

sub add_input_for_list {
    my $cb = shift;
    my ( $app, $res, $objs ) = @_;
    for my $data (@{$res->{objects}}) {
        my $id = $data->[0];
        my $entry = MT::Entry->load($id);
        next if !$entry;
        $data->[3] = format_ts( "%Y-%m-%d %k:%M:%S", $entry->authored_on, $entry->blog,
                $app->user ? $app->user->preferred_language : undef );
        $data->[4] = format_ts( "%Y-%m-%d %k:%M:%S", $entry->modified_on, $entry->blog,
                $app->user ? $app->user->preferred_language : undef );
#        my $author_id = $entry->author_id;
#        $data->[1] = '<input type="hidden" name="author_id_' . $id . '" value="' . $author_id . '" id="entry_author_id_' . $id . '">' . $data->[1];
    }
}

sub save_from_list_cd {
    my $app = shift;

    return $app->forward('filtered_list') if is_limit();

    $app->validate_magic or return;

    my $author = $app->user;
    my $type = $app->param('_type') || 'content_data';
    $type = 'author' if $type eq 'member';

    my $class = $app->model($type)
        or return $app->errtrans("Invalid parameter");

    my $perms;
    if ( $type eq 'content_data' ) {
        $perms = $app->permissions
            or return $app->permission_denied();
    }

    my $id = $app->param('id');
    return if !$id;

    require MT::Blog;
    my $blog_id = $app->param('blog_id');
    my $blog    = MT::Blog->load($blog_id)
        or return $app->error(
        $app->translate( 'Can\'t load blog #[_1].', $blog_id ) );
    my $content_type_id = $app->param('content_type_id');

    my $content_data = MT::ContentData->load( { blog_id => $blog_id, id => $id, content_type_id => $content_type_id } )
        || return $app->error(
        $app->translate( "No such [_1].", $class->class_label ) );

    if ($type ne 'author' && $blog->class eq 'blog') {
        return $app->error( $app->translate("Invalid parameter") )
            unless $content_data->blog_id == $blog_id;
    }

    my $data = $content_data->data;
    my $q = $app->param;
    foreach ( $q->param() ) {
        if (m/^customfield_(.*?)$/) {
            my $field_name = $1;
next if $q->param($_) =~ /none/;

            my $cf = MT::ContentField->load( $field_name ); 
            if ( $cf->type eq 'checkboxes' ||
                 $cf->type eq 'select_box' ) {
                my @values = split( /,/, $q->param($_) );
                $data->{ $field_name } = \@values;
            } 
            if ( $cf->type eq 'single_line_text' ||
                 $cf->type eq 'multi_line_text' ||
                 $cf->type eq 'number' ||
                 $cf->type eq 'url' ||
                 $cf->type eq 'radio_button' ||
                 $cf->type eq 'date_and_time' ||
                 $cf->type eq 'embedded_text' ) {
                $data->{ $field_name } = $q->param($_);
            }
            if ( $cf->type eq 'date_only' ) {
                $data->{ $field_name } = $q->param($_) . '000000';
            }
            if ( $cf->type eq 'time_only'  ) {
                $data->{ $field_name } = '19700101' . $q->param($_);
            }
            if ( $cf->type eq 'categories' ) {
                my @values = split( /,/, $q->param($_) );
                $data->{ $field_name } = \@values;
            }
            if ( $cf->type eq 'content_type' ) {
                my @values = split( /,/, $q->param($_) );
                $data->{ $field_name } = \@values;
            }
            if ( $cf->type eq 'tags' ) {
                my @a = split( /,\s*/, $q->param($_) );
                my @ids;
                for my $name (@a) {
                    my @iter = MT::Tag->load( { name => $name } );
                    if (!@iter) {
                        my $tag = MT::Tag->new( name => $name );
                        $tag->save;
                        push @ids, $tag->id;
                    } else {
                        for my $tag ( @iter ) {
                            push @ids, $tag->id;
                        }
                    }
                }
                $data->{ $field_name } = \@ids;
            }
            if ( $cf->type eq 'asset_image' ) {
                my @values = split( /,/, $q->param($_) );
                $data->{ $field_name } = \@values;
            } 
        }
    }

    $content_data->data($data) if $data;
    $content_data->save;

    return $app->forward('filtered_list');

#    CustomFields::App::CMS::CMSPostSave_customfield_objs('dummy', $app, $obj);
#
#    # for asset
#    require MT::Asset;
#    require MT::ObjectAsset;
#    my $include_asset_ids = $app->param('include_asset_ids');
#
#    if ($include_asset_ids) {
#        my @asset_ids;
#        if ($include_asset_ids =~ /\[(.*)\]/) {
#            @asset_ids = split( ',', $1 );
#        } else {
#            push @asset_ids, $include_asset_ids;
#        }
#        my $obj_assets        = ();
#        my @obj_assets        = MT::ObjectAsset->load(
#            { object_ds => 'entry', object_id => $obj->id } );
#        foreach my $obj_asset (@obj_assets) {
#            my $asset_id = $obj_asset->asset_id;
#            $obj_assets->{$asset_id} = 1;
#        }
#        my $seen = ();
#        foreach my $asset_id (@asset_ids) {
#            my $obj_asset = MT::ObjectAsset->load(
#                {   asset_id  => $asset_id,
#                    object_ds => 'entry',
#                    object_id => $obj->id
#                }
#            );
#            unless ($obj_asset) {
#                my $obj_asset = new MT::ObjectAsset;
#                $obj_asset->blog_id($blog_id);
#                $obj_asset->asset_id($asset_id);
#                $obj_asset->object_ds('entry');
#                $obj_asset->object_id( $obj->id );
#                $obj_asset->save;
#            }
#            $seen->{$asset_id} = 1;
#        }
#    }
#
#    return $app->forward('filtered_list');
}

sub make_content_actions {

    my $app = MT->instance;

    my $plugin = MT->component("PowerListingFieldEditor");
    my $edit_all = $plugin->translate('Edit all');

    my $data = {
       'entry' => {
           'reset_all_counter' => {
                mode  => 'view',
                class => 'icon-action',
                label => $edit_all,
                id    => 'action-edit-all',
                order => 10,
            },
       },
       'role' => {
            'create_role' => {
                mode  => 'view',
                class => 'icon-create',
                label => 'Create Role',
                icon  => 'ic_add',
                order => 100,
            }
        },
        'association' => {
            'grant_role' => {
                class => 'icon-create',
                label => 'Grant Permission',
                icon  => 'ic_add',
                mode  => 'dialog_grant_role',
                args  => {
                    _type => 'user',
                    type  => 'site',
                },
                return_args   => 1,
                permit_action => 'create_any_association',
                order         => 100,
                dialog        => 1,
            },
        },
        'member' => {
            'grant_role' => {
                class => 'icon-create',
                label => sub {
                    return $app->translate(
                        'Add a user to this [_1]',
                        lc $app->blog->class_label
                    );
                },
                icon => 'ic_add',
                mode => 'dialog_grant_role',
                args => sub {
                    if ( $app->blog->is_blog ) {
                        return {
                            type  => 'blog',
                            _type => 'user',
                        };
                    }
                    else {
                        return {
                            type  => 'website',
                            _type => 'user',
                        };
                    }
                },
                return_args => 1,
                order       => 100,
                dialog      => 1,
            },
        },
        'log' => {
            'reset_log' => {
                class       => 'icon-action',
                label       => 'Clear Activity Log',
                icon        => 'ic_setting',
                mode        => 'reset_log',
                order       => 100,
                confirm_msg => sub {
                    MT->translate(
                        'Are you sure you want to reset the activity log?');
                },
                permit_action => {
                    permit_action => 'reset_blog_log',
                    include_all   => 1,
                },
            },
            'download_log' => {
                class         => 'icon-download',
                label         => 'Download Log (CSV)',
                icon          => 'ic_download',
                mode          => 'export_log',
                order         => 200,
                permit_action => {
                    permit_action => 'export_blog_log',
                    include_all   => 1,
                    system_action => 'export_system_log',
                },
            },
        },
        'banlist' => {
            'ban_ip' => {
                class         => 'icon-create',
                label         => 'Add IP Address',
                id            => 'action-ban-ip',
                order         => 100,
                permit_action => 'save_banlist',
                condition     => sub {
                    MT->app && MT->app->param('blog_id');
                },
            },
        },
        'notification' => {
            'add_contact' => {
                class      => 'icon-create',
                label      => 'Add Contact',
                id         => 'action-create-contact',
                order      => 100,
                permission => 'edit_notifications',
                condition  => sub {
                    MT->app && MT->app->param('blog_id');
                },
            },
            'download_csv' => {
                class       => 'icon-download',
                label       => 'Download Address Book (CSV)',
                order       => 200,
                mode        => 'export_notification',
                return_args => 1,
                permission  => 'export_addressbook',
                condition   => sub {
                    MT->app && MT->app->param('blog_id');
                }
            },
        },

        %{ PowerListingFieldEditor::CMS::make_contentdata_content_actions() },
    };

    my $core = MT->component("core");
    $core->registry( "applications", "cms", "content_actions", $data );
}

sub make_contentdata_content_actions {

    my $plugin = MT->component("PowerListingFieldEditor");
    my $edit_all = $plugin->translate('Edit all');

    my $iter            = MT::ContentType->load_iter;
    my $content_actions = {};
    while ( my $ct = $iter->() ) {
        my $key = 'content_data.content_data_' . $ct->id;
        $content_actions->{$key} = {
            new => {
                label => sub {
                    MT->translate( 'Create new [_1]', $ct->name );
                },
                icon  => 'ic_add',
                order => 100,
                mode  => 'view',
                args  => {
                    blog_id         => $ct->blog_id,
                    content_type_id => $ct->id,
                },
                class => 'icon-create',
            },
            'reset_all_counter' => {
                mode  => 'view',
                class => 'icon-action',
                label => $edit_all,
                id    => 'action-edit-all',
                order => 10,
            },
        };
    }
    $content_actions;
}

1;
